@@ -1,20 +1,25 @@
 """Constants for the executors."""
+
 from pathlib import Path
 
-DATA_LOCAL_VOLUME = Path("/data_local")
-DATA_VOLUME = Path("/data")
-DATA_ALL_VOLUME = Path("/data_all")
-UPLOAD_VOLUME = Path("/upload")
 SECRETS_VOLUME = Path("/secrets")
-SETTINGS_VOLUME = Path("/settings")
 SOCKETS_VOLUME = Path("/sockets")
 INPUTS_VOLUME = Path("/inputs")
+PROCESSING_VOLUME = Path("/processing")
 
 CONTAINER_TIMEOUT = 600
-# Where socket files are stored inside container.
+
+# Relative path to socket files inside sockets volume.
 COMMUNICATION_PROCESSING_SOCKET = "_socket1.s"
 SCRIPT_SOCKET = "_socket2.s"
-# Used to upload files from processing to communication container.
 UPLOAD_FILE_SOCKET = "_upload_socket.s"
 
 TMPDIR = ".tmp"
+
+INPUTS_VOLUME_NAME = "input"
+PROCESSING_VOLUME_NAME = "processing"
+RUNTIME_VOLUME_NAME = "runtime"
+SECRETS_VOLUME_NAME = "secrets"
+SOCKETS_VOLUME_NAME = "sockets"
+
+BOOTSTRAP_PYTHON_RUNTIME = "bootstrap_python_runtime.py"
