@@ -4,15 +4,13 @@
     :members:
 
 """
+
 # pylint: disable=logging-format-interpolation
 import asyncio
 import copy
 import functools
-import json
 import logging
 import os
-import random
-import string
 import tempfile
 import time
 from contextlib import suppress
@@ -27,7 +25,6 @@ from ..connectors.baseconnector import BaseStorageConnector
 from ..global_settings import LOCATION_SUBPATH, PROCESS_META, SETTINGS
 from ..local.run import FlowExecutor as LocalFlowExecutor
 from ..protocol import ExecutorFiles
-from .seccomp import SECCOMP_POLICY
 
 # Limits of containers' access to memory. We set the limit to ensure
 # processes are stable and do not get killed by OOM signal.
@@ -38,11 +35,6 @@ DOCKER_MEMORY_SWAPPINESS = 1
 logger = logging.getLogger(__name__)
 
 
-def _random_string(size: int = 5, chars=string.ascii_lowercase + string.digits):
-    """Generate and return random string."""
-    return "".join(random.choice(chars) for x in range(size))
-
-
 def retry(
     max_retries: int = 3,
     retry_exceptions: Tuple[Type[Exception], ...] = (
@@ -298,7 +290,7 @@ class FlowExecutor(LocalFlowExecutor):
 
         # Create mount points for tools.
         mount_points += [
-            ({"path": Path(tool)}, Path("/usr/local/bin/resolwe") / str(index), True)
+            ({"path": Path(tool)}, self.tools_paths_prefix / str(index), True)
             for index, tool in enumerate(self.get_tools_paths())
         ]
         return dict([self._new_volume(*mount_point) for mount_point in mount_points])
@@ -310,12 +302,14 @@ class FlowExecutor(LocalFlowExecutor):
         )
         memory_swap = int(memory * DOCKER_MEMORY_SWAP_RATIO)
 
-        # By default the container is connected to the bridge network.
+        # By default the communication container is connected to the bridge network.
         network = SETTINGS.get("FLOW_EXECUTOR", {}).get("NETWORK", "bridge")
 
-        security_options = []
-        if not SETTINGS.get("FLOW_DOCKER_DISABLE_SECCOMP", False):
-            security_options.append(f"seccomp={json.dumps(SECCOMP_POLICY)}")
+        # The network in the processing container depends on the process requirements.
+        if "network" in self.resources:
+            network_mode = f"container:{self.container_name}-communicator"
+        else:
+            network_mode = "none"
 
         processing_image = self.requirements.get(
             "image",
@@ -361,19 +355,17 @@ class FlowExecutor(LocalFlowExecutor):
                 for connector in connectors.values()
                 if connector.mountable
             ),
+            "TOOLS_PATHS": ":".join(
+                f"{self.tools_paths_prefix / str(i)}"
+                for i in range(len(self.get_tools_paths()))
+            ),
         }
         with suppress(RuntimeError):
             environment["UPLOAD_DIR"] = get_upload_dir()
 
         autoremove = SETTINGS.get("FLOW_DOCKER_AUTOREMOVE", False)
 
-        # Add random string between container name and init. Since check for
-        # existing stdout file has been moved inside init container we should
-        # use different containers name in case one init contaner is still
-        # running when another one is fired (or when containers are not purged
-        # automatically): otherwise executor will fail to start the init
-        # container due to name clash.
-        init_container_name = f"{self.container_name}-{_random_string()}-init"
+        init_container_name = f"{self.container_name}-init"
 
         init_arguments = {
             "auto_remove": autoremove,
@@ -402,7 +394,6 @@ class FlowExecutor(LocalFlowExecutor):
             "mem_reservation": "200m",
             "network": network,
             "cap_drop": ["all"],
-            "security_opt": security_options,
             "user": f"{os.getuid()}:{os.getgid()}",
             "environment": environment,
         }
@@ -411,7 +402,7 @@ class FlowExecutor(LocalFlowExecutor):
             "volumes": self._processing_volumes(),
             "command": ["python3", "/start.py"],
             "image": processing_image,
-            "network_mode": f"container:{self.container_name}-communicator",
+            "network_mode": network_mode,
             "working_dir": os.fspath(self.processing_working_dir),
             "detach": True,
             "cpu_quota": self.process["resource_limits"]["cores"] * (10**6),
@@ -421,7 +412,6 @@ class FlowExecutor(LocalFlowExecutor):
             "memswap_limit": f"{memory_swap}m",
             "name": self.container_name,
             "cap_drop": ["all"],
-            "security_opt": security_options,
             "user": f"{os.getuid()}:{os.getgid()}",
             "ulimits": ulimits,
             "environment": environment,
