@@ -3,10 +3,9 @@
 .. autoclass:: resolwe.flow.executors.null.run.FlowExecutor
 
 """
+
 import logging
 
-from ..manager_commands import send_manager_command
-from ..protocol import ExecutorProtocol
 from ..run import BaseFlowExecutor
 
 logger = logging.getLogger(__name__)
@@ -21,6 +20,6 @@ class FlowExecutor(BaseFlowExecutor):
 
     name = "null"
 
-    async def run(self, data_id, script, log_file, json_file):
+    async def run(self):
         """Do nothing :)."""
-        await send_manager_command(ExecutorProtocol.FINISH)
+        pass
