@@ -7,16 +7,26 @@ Consumer
 Manager Channels consumer.
 
 """
+
 import asyncio
 import logging
+import os
+from contextlib import suppress
+from pathlib import Path
 
 from channels.consumer import AsyncConsumer
+from channels.db import database_sync_to_async
 from channels.layers import get_channel_layer
 from channels.testing import ApplicationCommunicator
 
+from django.db import connection
+
+from resolwe.utils import BraceMessage as __
+
 from . import state
 
 logger = logging.getLogger(__name__)
+CHANNEL_HEALTH_CHECK = os.environ.get("HOSTNAME")
 
 
 async def send_event(message):
@@ -43,13 +53,6 @@ async def run_consumer(timeout=None):
         "channel": manager_channel,
     }
     manager_app = ApplicationCommunicator(ManagerConsumer(), manager_scope)
-
-    listener_channel = state.LISTENER_CONTROL_CHANNEL
-    listener_scope = {
-        "channel": listener_channel,
-    }
-    listener_app = ApplicationCommunicator(ListenerConsumer(), listener_scope)
-
     channel_layer = get_channel_layer()
 
     async def _consume_loop(channel, scope, app):
@@ -60,21 +63,14 @@ async def run_consumer(timeout=None):
             await app.send_input(message)
             message = await channel_layer.receive(channel)
 
-    awaitables = [
-        asyncio.ensure_future(
-            _consume_loop(manager_channel, manager_scope, manager_app)
-        ),
-        asyncio.ensure_future(
-            _consume_loop(listener_channel, listener_scope, listener_app)
-        ),
-    ]
+    consume_future = asyncio.ensure_future(
+        _consume_loop(manager_channel, manager_scope, manager_app)
+    )
 
-    await asyncio.wait(awaitables, timeout=timeout)
-    for awaitable in awaitables:
-        awaitable.cancel()
+    with suppress(asyncio.TimeoutError):
+        await asyncio.wait_for(consume_future, timeout=timeout)
 
     await manager_app.wait()
-    await listener_app.wait()
 
 
 async def exit_consumer():
@@ -83,7 +79,6 @@ async def exit_consumer():
         "type": "_resolwe_manager_quit",
     }
     await get_channel_layer().send(state.MANAGER_CONTROL_CHANNEL, packet)
-    await get_channel_layer().send(state.LISTENER_CONTROL_CHANNEL, packet)
 
 
 class ManagerConsumer(AsyncConsumer):
@@ -105,16 +100,29 @@ class ManagerConsumer(AsyncConsumer):
             logger.exception("control_event exception.")
 
 
-class ListenerConsumer(AsyncConsumer):
-    """Channels consumer for handling manager events."""
+class HealtCheckConsumer(AsyncConsumer):
+    """Channels consumer for handling health-check events."""
 
-    def __init__(self, *args, **kwargs):
-        """Initialize a consumer instance with the given manager."""
-        # This import is local in order to avoid startup import loops.
-        from . import listener
+    async def health_check(self, message: dict):
+        """Perform health check.
 
-        self.listener = listener
+        We are testing the channels layer and database layer. The channels
+        layer is already functioning if this method is called so we have to
+        perform database check.
 
-    async def terminate_worker_event(self, message):
-        """Forward control events to the manager dispatcher."""
-        await self.listener.terminate_worker(message["identity"])
+        If the check is successfull touch the file specified in the channels
+        message.
+        """
+        logger.debug(__("Performing health check with message {}.", message))
+        path = Path(message["file"])
+
+        if await database_sync_to_async(self.check_database, thread_sensitive=False)():
+            logger.debug("Health check passed.")
+            path.touch(exist_ok=True)
+
+    def check_database(self) -> bool:
+        """Perform a simple database check."""
+        with connection.cursor() as cursor:
+            cursor.execute("SELECT 1;")
+            result = cursor.fetchone()[0]
+            return result == 1
