@@ -5,18 +5,23 @@ Kubernetes Connector
 ====================
 
 """
+
 import hashlib
 import json
 import logging
 import os
+import random
 import re
+import string
 import time
 from contextlib import suppress
+from enum import Enum
 from importlib.util import find_spec
 from pathlib import Path
 from typing import Any, Dict, Iterable, Tuple
 
 import kubernetes
+import redis
 
 from django.conf import settings
 from django.db.models import Sum
@@ -24,7 +29,7 @@ from django.db.models.functions import Coalesce
 
 from resolwe.flow.executors import constants
 from resolwe.flow.models import Data, DataDependency, Process
-from resolwe.flow.utils import get_apps_tools
+from resolwe.flow.utils.decorators import retry
 from resolwe.storage import settings as storage_settings
 from resolwe.storage.connectors import connectors
 from resolwe.storage.connectors.baseconnector import BaseStorageConnector
@@ -32,6 +37,13 @@ from resolwe.utils import BraceMessage as __
 
 from .base import BaseConnector
 
+redis_server = redis.from_url(
+    getattr(settings, "REDIS_CONNECTION_STRING", "redis://localhost")
+)
+
+# Redis random postfix cache.
+REDIS_POSTFIX_KEY = "workload_kubernetes_random_postfix"
+
 # TODO: is this really needed?
 # Limits of containers' access to memory. We set the limit to ensure
 # processes are stable and do not get killed by OOM signal.
@@ -70,9 +82,9 @@ def get_upload_dir() -> str:
     raise RuntimeError("No mountable upload connector is defined.")
 
 
-def unique_volume_name(base_name: str, data_id: int) -> str:
+def unique_volume_name(base_name: str, data_id: int, postfix: str) -> str:
     """Get unique persistent volume claim name."""
-    return f"{base_name}-{data_id}"
+    return f"{base_name}-{data_id}-{postfix}"
 
 
 def sanitize_kubernetes_label(label: str, trim_end: bool = True) -> str:
@@ -93,11 +105,21 @@ def sanitize_kubernetes_label(label: str, trim_end: bool = True) -> str:
     return sanitized_label
 
 
+class ConfigLocation(Enum):
+    """The enum specifying where to read the configuration from."""
+
+    INCLUSTER = "incluster"
+    KUBECTL = "kubectl"
+
+
 class Connector(BaseConnector):
     """Kubernetes-based connector for job execution."""
 
     def __init__(self):
         """Initialize."""
+        self._config_location = ConfigLocation(
+            settings.KUBERNETES_DISPATCHER_CONFIG_LOCATION
+        )
         self._initialize_variables()
 
     def _initialize_variables(self):
@@ -108,12 +130,17 @@ class Connector(BaseConnector):
         self.kubernetes_namespace = getattr(settings, "KUBERNETES_SETTINGS", {}).get(
             "namespace", "default"
         )
+        self.tools_path_prefix = Path("/usr/local/bin/resolwe")
 
     def _prepare_environment(
-        self, data: Data, listener_connection: Tuple[str, str, str]
+        self,
+        data: Data,
+        listener_connection: Tuple[str, str, str],
+        tools_configmaps: Dict[str, str],
     ) -> list:
         """Prepare environmental variables."""
         host, port, protocol = listener_connection
+        processing_uid, processing_gid = self._get_processing_uid_gid()
         environment = {
             "LISTENER_SERVICE_HOST": host,
             "LISTENER_SERVICE_PORT": port,
@@ -124,14 +151,18 @@ class Connector(BaseConnector):
             ),
             "RUNNING_IN_CONTAINER": 1,
             "RUNNING_IN_KUBERNETES": 1,
-            "GENIALIS_UID": os.getuid(),
-            "GENIALIS_GID": os.getgid(),
+            "GENIALIS_UID": processing_uid,
+            "GENIALIS_GID": processing_gid,
             "DESCRIPTOR_CHUNK_SIZE": 100,
             "MOUNTED_CONNECTORS": ",".join(
                 connector.name
                 for connector in connectors.values()
                 if connector.mountable
             ),
+            "TOOLS_PATHS": ":".join(
+                f"{self.tools_path_prefix / tools_name}"
+                for tools_name in tools_configmaps
+            ),
         }
         with suppress(RuntimeError):
             environment["UPLOAD_DIR"] = get_upload_dir()
@@ -162,7 +193,7 @@ class Connector(BaseConnector):
                     _request_timeout=KUBERNETES_TIMEOUT,
                 )
 
-    def _get_tools_configmaps(self, core_api):
+    def _get_tools_configmaps(self, core_api) -> Dict[str, str]:
         """Get and return configmaps for tools."""
         description_configmap_name = (
             getattr(settings, "KUBERNETES_TOOLS_CONFIGMAPS", None) or "tools-configmaps"
@@ -170,7 +201,7 @@ class Connector(BaseConnector):
         configmap = core_api.read_namespaced_config_map(
             name=description_configmap_name, namespace=self.kubernetes_namespace
         )
-        return configmap.data.values()
+        return configmap.data
 
     def _get_files_configmap_name(self, location_subpath: Path, core_api: Any):
         """Get or create configmap for files.
@@ -179,10 +210,11 @@ class Connector(BaseConnector):
         This is done using configmap which is mostly static. This command
         returns description for this configmap.
         """
+        processing_uid, processing_gid = self._get_processing_uid_gid()
         passwd_content = "root:x:0:0:root:/root:/bin/bash\n"
-        passwd_content += f"user:x:{os.getuid()}:{os.getgid()}:user:{os.fspath(constants.PROCESSING_VOLUME)}:/bin/bash\n"
+        passwd_content += f"user:x:{processing_uid}:{processing_gid}:user:{os.fspath(constants.PROCESSING_VOLUME)}:/bin/bash\n"
         group_content = "root:x:0:\n"
-        group_content += f"user:x:{os.getgid()}:user\n"
+        group_content += f"user:x:{processing_gid}:user\n"
 
         data = dict()
         modules = {
@@ -212,14 +244,21 @@ class Connector(BaseConnector):
         self._create_configmap_if_needed(configmap_name, data, core_api)
         return configmap_name
 
-    def _volumes(self, data_id: int, location_subpath: Path, core_api: Any) -> list:
+    def _volumes(
+        self,
+        data_id: int,
+        location_subpath: Path,
+        core_api: Any,
+        tools_configmaps: Dict[str, str],
+        random_postfix: str,
+    ) -> list:
         """Prepare all volumes."""
 
         def volume_from_config(volume_name: str, volume_config: dict):
             """Get configuration for kubernetes for given volume."""
             claim_name = volume_config["config"]["name"]
             if self._should_create_pvc(volume_config):
-                claim_name = unique_volume_name(claim_name, data_id)
+                claim_name = unique_volume_name(claim_name, data_id, random_postfix)
             if volume_config["type"] == "persistent_volume":
                 return {
                     "name": volume_name,
@@ -247,11 +286,11 @@ class Connector(BaseConnector):
                 "configMap": {"name": files_configmap_name},
             }
         ]
-        tools_configmaps = self._get_tools_configmaps(core_api)
-        for index, configmap_name in enumerate(tools_configmaps):
+
+        for tools_name, configmap_name in tools_configmaps.items():
             volumes.append(
                 {
-                    "name": f"tools-{index}",
+                    "name": f"tools-{tools_name}",
                     "configMap": {"name": configmap_name, "defaultMode": 0o755},
                 }
             )
@@ -264,7 +303,7 @@ class Connector(BaseConnector):
 
         for storage_name, connector in get_mountable_connectors():
             claim_name = connector.config.get("persistent_volume_claim", None)
-            volume_data = {
+            volume_data: Dict[str, Any] = {
                 "name": connector.name,
             }
             if claim_name:
@@ -341,7 +380,7 @@ class Connector(BaseConnector):
         return mount_points
 
     def _processing_mountpoints(
-        self, location_subpath: Path, execution_engine_name: str
+        self, location_subpath: Path, tools_configmaps: Dict[str, str]
     ):
         """Mountpoints for processing container.
 
@@ -420,23 +459,35 @@ class Connector(BaseConnector):
                 "readOnly": True,
             },
         ]
-        for tool_index in range(len(get_apps_tools())):
+        for tool_name in tools_configmaps:
             mount_points.append(
                 {
-                    "name": f"tools-{tool_index}",
-                    "mountPath": f"/usr/local/bin/resolwe/{tool_index}",
+                    "name": f"tools-{tool_name}",
+                    "mountPath": f"{self.tools_path_prefix / tool_name}",
                 }
             )
         return mount_points
 
+    def _create_labels(self, data: Data, job_type: str) -> dict[str, str]:
+        """Create labels for kubernetes objects."""
+        return {
+            "application": "resolwe",
+            "data_id": str(data.pk),
+            "process": sanitize_kubernetes_label(data.process.slug),
+            "job_type": sanitize_kubernetes_label(job_type),
+        }
+
     def _persistent_volume_claim(
-        self, claim_name: str, size: int, volume_config: Dict
+        self, claim_name: str, size: int, volume_config: Dict, labels: dict[str, str]
     ) -> Dict[str, Any]:
         """Prepare claim for persistent volume."""
         return {
             "apiVersion": "v1",
             "kind": "PersistentVolumeClaim",
-            "metadata": {"name": claim_name},
+            "metadata": {
+                "name": claim_name,
+                "labels": labels,
+            },
             "spec": {
                 "accessModes": ["ReadWriteOnce"],
                 "storageClassName": volume_config.get("storageClassName", "gp2"),
@@ -448,6 +499,12 @@ class Connector(BaseConnector):
             },
         }
 
+    def _get_processing_uid_gid(self) -> Tuple[int, int]:
+        """Get the UID and GID of the processing container."""
+        return getattr(settings, "FLOW_PROCESSING_USER", os.getuid()), getattr(
+            settings, "FLOW_PROCESSING_GROUP", os.getgid()
+        )
+
     def _data_inputs_size(self, data: Data, safety_buffer: int = 2**30) -> int:
         """Get the size of data inputs.
 
@@ -486,55 +543,135 @@ class Connector(BaseConnector):
                 return f"{mapper[key]}{image_name[len(key):]}"
         return image_name
 
+    @retry(
+        logger=logger,
+        max_retries=5,
+        retry_exceptions=(kubernetes.config.config_exception.ConfigException,),
+        cleanup_callback=lambda: kubernetes.config.kube_config._cleanup_temp_files(),
+    )
+    def _load_kubernetes_config(self):
+        """Load the kubernetes configuration.
+
+        :raises kubernetes.config.config_exception.ConfigException: when the
+            configuration could not be read.
+        """
+        (
+            kubernetes.config.load_incluster_config()
+            if self._config_location == ConfigLocation.INCLUSTER
+            else kubernetes.config.load_kube_config()
+        )
+
+    def optimize_job_scheduling(self, data: Data, job_description: dict):
+        """Optimize the scheduling by modifying the job description.
+
+        Moodify the job description to be a better fit on the scheduler. Currently this
+        is used to schedule based on the process scheduling class.
+        """
+        scheduling_class_name = dict(Process.SCHEDULING_CLASS_CHOICES)[
+            data.process.scheduling_class
+        ].lower()
+        if kubernetes_affinity := getattr(settings, "FLOW_KUBERNETES_AFFINITY", None):
+            scheduling_class_affinity = kubernetes_affinity.get("scheduling_class", {})
+            affinity_settings = scheduling_class_affinity.get(scheduling_class_name, {})
+            if match_expressions := [
+                {"key": key, "operator": "In", "values": values}
+                for key, values in affinity_settings.items()
+            ]:
+                job_description["spec"]["template"]["spec"]["affinity"] = {
+                    "nodeAffinity": {
+                        "requiredDuringSchedulingIgnoredDuringExecution": {
+                            "nodeSelectorTerms": [
+                                {"matchExpressions": match_expressions}
+                            ]
+                        }
+                    }
+                }
+
+    def _get_overcommit_factors(self, data: Data) -> dict:
+        """Get the overcommit settings for CPU and memory.
+
+        The returned dict is of the form:
+        { 'cpu': 0.8, 'memory': 0.8 }
+
+        The numbers indicite that we will only request 80% of CPU and RAM requested in
+        the process.
+        """
+        default = {"cpu": 0.8, "memory": 0.8}
+        job_type = dict(Process.SCHEDULING_CLASS_CHOICES)[data.process.scheduling_class]
+        return default | settings.FLOW_KUBERNETES_OVERCOMMIT.get(job_type, {})
+
     def start(self, data: Data, listener_connection: Tuple[str, str, str]):
         """Start process execution.
 
         Construct kubernetes job description and pass it to the kubernetes.
         """
-        container_environment = self._prepare_environment(data, listener_connection)
-
-        location_subpath = Path(data.location.subpath)
-
-        # Create kubernetes API every time otherwise it will time out
-        # eventually and raise API exception.
         try:
-            kubernetes.config.load_kube_config()
-        except kubernetes.config.config_exception.ConfigException:
-            kubernetes.config.load_incluster_config()
+            self._load_kubernetes_config()
+        except kubernetes.config.config_exception.ConfigException as exception:
+            logger.exception("Could not load the kubernetes configuration.")
+            raise exception
 
         batch_api = kubernetes.client.BatchV1Api()
         core_api = kubernetes.client.CoreV1Api()
 
+        tools_configmaps = self._get_tools_configmaps(core_api)
+        container_environment = self._prepare_environment(
+            data, listener_connection, tools_configmaps
+        )
+        location_subpath = Path(data.location.subpath)
+
+        # Set random postfix string for volume and pod names.
+        random_postfix = "".join(
+            random.choices(string.ascii_lowercase + string.digits, k=5)
+        )
+
+        # Store the random postfix in redis for later use in the cleanup method.
+        redis_server.hset(REDIS_POSTFIX_KEY, data.id, random_postfix)
+
         container_name_prefix = (
             getattr(settings, "FLOW_EXECUTOR", {})
             .get("CONTAINER_NAME_PREFIX", "resolwe")
             .replace("_", "-")
             .lower()
         )
-        container_name = self._generate_container_name(container_name_prefix, data.pk)
+
+        container_name = self._generate_container_name(
+            container_name_prefix, data.pk, random_postfix
+        )
 
         # Set resource limits.
         requests = dict()
         limits = data.get_resource_limits()
+        overcommit_factors = self._get_overcommit_factors(data)
 
-        requests["cpu"] = limits.pop("cores")
-        limits["cpu"] = requests["cpu"] + 1
-        # Overcommit CPU by 20%.
-        requests["cpu"] *= 0.8
+        requests["cpu"] = limits["cores"] * overcommit_factors["cpu"]
+        limits["cpu"] = limits.pop("cores") + 1
 
         # The memory in the database is stored in megabytes but the kubertenes
         # requires memory in bytes.
-        # We request 10% less memory than stored in the database and set limit
-        # at 10% more plus KUBERNETES_MEMORY_HARD_LIMIT_BUFFER. The processes
-        # usually require 16GB, 32GB... and since the node usualy has 64GB of
-        # memory and some of it is consumed by the system processes only one
-        # process process that requires 32GB can run on a node instead of 2.
+        # We request less memory than stored in the database and set limit at 10% more
+        # plus KUBERNETES_MEMORY_HARD_LIMIT_BUFFER. The processes usually require 16GB,
+        # 32GB... and since the node usualy has 64GB of memory and some of it is
+        # consumed by the system processes only one process process that requires 32GB
+        # can run on a node instead of 2.
 
-        requests["memory"] = 0.9 * limits["memory"]
+        requests["memory"] = limits["memory"] * overcommit_factors["memory"]
         limits["memory"] = 1.1 * limits["memory"] + KUBERNETES_MEMORY_HARD_LIMIT_BUFFER
         limits["memory"] *= 2**20  # 2 ** 20 = mebibyte
         requests["memory"] *= 2**20
 
+        # Get the limits and requests for the communicator container.
+        communicator_limits = getattr(
+            settings,
+            "FLOW_KUBERNETES_COMMUNICATOR_LIMITS",
+            {"memory": "256M", "cpu": 0.1},
+        )
+        communicator_requests = getattr(
+            settings,
+            "FLOW_KUBERNETES_COMMUNICATOR_REQUESTS",
+            {"memory": "256M", "cpu": 0.1},
+        )
+
         resources = data.process.requirements.get("resources", {})
         network = "bridge"
         use_host_network = False
@@ -546,9 +683,10 @@ class Connector(BaseConnector):
             use_host_network = network == "host"
 
         # Generate and set seccomp policy to limit syscalls.
+        processing_uid, processing_gid = self._get_processing_uid_gid()
         security_context = {
-            "runAsUser": os.getuid(),
-            "runAsGroup": os.getgid(),
+            "runAsUser": processing_uid,
+            "runAsGroup": processing_gid,
             "allowPrivilegeEscalation": False,
             "privileged": False,
             "capabilities": {"drop": ["ALL"]},
@@ -559,19 +697,18 @@ class Connector(BaseConnector):
         # Do not evict job from node.
         annotations["cluster-autoscaler.kubernetes.io/safe-to-evict"] = "false"
 
-        if not getattr(settings, "FLOW_DOCKER_DISABLE_SECCOMP", False):
-            # The path is a relative path in the kubelet root
-            # directory:
-            # <seccomp_root>/<path>, where <seccomp_root> is defined via the
-            # --seccomp-profile-root flag on the Kubelet. If the
-            # --seccomp-profile-root flag is not defined, the default path will
-            # be used, which is <root-dir>/seccomp where <root-dir> is
-            # specified by the --root-dir flag.
-            # https://kubernetes.io/docs/concepts/policy/pod-security-policy/
-            #
-            # The file is transfered to kubelets with daemonset ? Currently I
-            # mount my /tmp directory to the /seccomp directory in minikube.
-            annotations["seccomp.security.alpha.kubernetes.io/pod"] = "runtime/default"
+        # The path is a relative path in the kubelet root
+        # directory:
+        # <seccomp_root>/<path>, where <seccomp_root> is defined via the
+        # --seccomp-profile-root flag on the Kubelet. If the
+        # --seccomp-profile-root flag is not defined, the default path will
+        # be used, which is <root-dir>/seccomp where <root-dir> is
+        # specified by the --root-dir flag.
+        # https://kubernetes.io/docs/concepts/policy/pod-security-policy/
+        #
+        # The file is transfered to kubelets with daemonset ? Currently I
+        # mount my /tmp directory to the /seccomp directory in minikube.
+        annotations["seccomp.security.alpha.kubernetes.io/pod"] = "runtime/default"
 
         mapper = getattr(settings, "FLOW_CONTAINER_IMAGE_MAP", {})
         communicator_image = getattr(
@@ -596,28 +733,9 @@ class Connector(BaseConnector):
             processing_container_image, mapper
         )
 
-        affinity = {}
-        kubernetes_affinity = getattr(settings, "FLOW_KUBERNETES_AFFINITY", None)
-        if kubernetes_affinity:
-            affinity = {
-                "nodeAffinity": {
-                    "requiredDuringSchedulingIgnoredDuringExecution": {
-                        "nodeSelectorTerms": [
-                            {
-                                "matchExpressions": [
-                                    {
-                                        "key": "nodegroup",
-                                        "operator": "In",
-                                        "values": [kubernetes_affinity],
-                                    }
-                                ]
-                            }
-                        ]
-                    }
-                }
-            }
-
+        pull_policy = getattr(settings, "FLOW_KUBERNETES_PULL_POLICY", "Always")
         job_type = dict(Process.SCHEDULING_CLASS_CHOICES)[data.process.scheduling_class]
+        labels = self._create_labels(data, job_type)
         job_description = {
             "apiVersion": "batch/v1",
             "kind": "Job",
@@ -632,25 +750,26 @@ class Connector(BaseConnector):
                 "template": {
                     "metadata": {
                         "name": sanitize_kubernetes_label(container_name),
-                        "labels": {
-                            "app": "resolwe",
-                            "data_id": str(data.pk),
-                            "process": sanitize_kubernetes_label(data.process.slug),
-                            "job_type": sanitize_kubernetes_label(job_type),
-                        },
+                        "labels": labels,
                         "annotations": annotations,
                     },
                     "spec": {
-                        "affinity": affinity,
+                        "affinity": {},
                         "hostNetwork": use_host_network,
-                        "volumes": self._volumes(data.id, location_subpath, core_api),
+                        "volumes": self._volumes(
+                            data.id,
+                            location_subpath,
+                            core_api,
+                            tools_configmaps,
+                            random_postfix,
+                        ),
                         "initContainers": [
                             {
                                 "name": sanitize_kubernetes_label(
                                     f"{container_name}-init"
                                 ),
                                 "image": communicator_image,
-                                "imagePullPolicy": "Always",
+                                "imagePullPolicy": pull_policy,
                                 "workingDir": "/",
                                 "command": ["/usr/local/bin/python3"],
                                 "args": ["-m", "executors.init_container"],
@@ -667,12 +786,11 @@ class Connector(BaseConnector):
                                 "securityContext": security_context,
                                 "env": container_environment,
                                 "workingDir": os.fspath(constants.PROCESSING_VOLUME),
-                                "imagePullPolicy": "Always",
+                                "imagePullPolicy": pull_policy,
                                 "command": ["/usr/bin/python3"],
                                 "args": ["/processing.py"],
                                 "volumeMounts": self._processing_mountpoints(
-                                    location_subpath,
-                                    data.process.run.get("language", None),
+                                    location_subpath, tools_configmaps
                                 ),
                             },
                             {
@@ -680,10 +798,10 @@ class Connector(BaseConnector):
                                     f"{container_name}-communicator"
                                 ),
                                 "image": communicator_image,
-                                "imagePullPolicy": "Always",
+                                "imagePullPolicy": pull_policy,
                                 "resources": {
-                                    "limits": {"cpu": 1, "memory": "1024M"},
-                                    "requests": {"memory": "256M", "cpu": 0.1},
+                                    "limits": communicator_limits,
+                                    "requests": communicator_requests,
                                 },
                                 "securityContext": security_context,
                                 "env": container_environment,
@@ -700,14 +818,17 @@ class Connector(BaseConnector):
                 "backoffLimit": 0,
             },
         }
+        self.optimize_job_scheduling(data, job_description)
         start_time = time.time()
 
         processing_name = constants.PROCESSING_VOLUME_NAME
         input_name = constants.INPUTS_VOLUME_NAME
+        created_claim_names = []
         if self._should_create_pvc(storage_settings.FLOW_VOLUMES[processing_name]):
             claim_name = unique_volume_name(
                 storage_settings.FLOW_VOLUMES[processing_name]["config"]["name"],
                 data.id,
+                random_postfix,
             )
             claim_size = limits.pop("storage", 200) * (2**30)  # Default 200 gibibytes
             core_api.create_namespaced_persistent_volume_claim(
@@ -715,29 +836,34 @@ class Connector(BaseConnector):
                     claim_name,
                     claim_size,
                     storage_settings.FLOW_VOLUMES[processing_name]["config"],
+                    labels,
                 ),
                 namespace=self.kubernetes_namespace,
                 _request_timeout=KUBERNETES_TIMEOUT,
             )
+            created_claim_names.append(claim_name)
         if input_name in storage_settings.FLOW_VOLUMES:
             if self._should_create_pvc(storage_settings.FLOW_VOLUMES[input_name]):
                 claim_size = self._data_inputs_size(data)
                 claim_name = unique_volume_name(
                     storage_settings.FLOW_VOLUMES[input_name]["config"]["name"],
                     data.id,
+                    random_postfix,
                 )
                 core_api.create_namespaced_persistent_volume_claim(
                     body=self._persistent_volume_claim(
                         claim_name,
                         claim_size,
                         storage_settings.FLOW_VOLUMES[input_name]["config"],
+                        labels,
                     ),
                     namespace=self.kubernetes_namespace,
                     _request_timeout=KUBERNETES_TIMEOUT,
                 )
+                created_claim_names.append(claim_name)
 
         logger.debug(f"Creating namespaced job: {job_description}")
-        batch_api.create_namespaced_job(
+        job = batch_api.create_namespaced_job(
             body=job_description,
             namespace=self.kubernetes_namespace,
             _request_timeout=KUBERNETES_TIMEOUT,
@@ -747,13 +873,49 @@ class Connector(BaseConnector):
             "It took {:.2f}s to send config to kubernetes".format(end_time - start_time)
         )
 
-    def _generate_container_name(self, prefix: str, data_id: int):
+        # Patch the PVC-s with owner references to the job.
+        patch = [
+            {
+                "op": "add",
+                "path": "/metadata/ownerReferences",
+                "value": [
+                    {
+                        "apiVersion": "batch/v1",
+                        "kind": "Job",
+                        "name": job.metadata.name,
+                        "uid": job.metadata.uid,
+                    }
+                ],
+            }
+        ]
+        for claim_name in created_claim_names:
+            # Patch the PVC with ownerReference. This ensures the PVC claim gets deleted
+            # when the job has completed or failed. The PVCs are deleted after
+            # ttlSecondsAfterFinished seconds, declared in Job spec.
+            try:
+                core_api.patch_namespaced_persistent_volume_claim(
+                    name=claim_name, namespace=self.kubernetes_namespace, body=patch
+                )
+            except Exception as error:
+                # This operation is not critical for the job to run, so we log the
+                # error and continue. The job will still run, but the PVC will not be
+                # deleted automatically after the job has finished.
+                logger.exception(
+                    __(
+                        "Kubernetes owner patch for pvc {} failed: '{}'.",
+                        claim_name,
+                        error,
+                    )
+                )
+
+    def _generate_container_name(self, prefix: str, data_id: int, postfix: str):
         """Generate unique container name.
 
         Name of the kubernetes container should contain only lower case
         alpfanumeric characters and dashes. Underscores are not allowed.
         """
-        return "{}-{}".format(prefix, data_id)
+        # Append random string to make it safe for restart.
+        return f"{prefix}-{data_id}-{postfix}"
 
     def submit(self, data: Data, argv):
         """Run process.
@@ -763,7 +925,16 @@ class Connector(BaseConnector):
         """
         (host, port, protocol) = argv[-1].rsplit(" ", maxsplit=3)[-3:]
         self._initialize_variables()
-        self.start(data, (host, port, protocol))
+        try:
+            self.start(data, (host, port, protocol))
+        except Exception as error:
+            error_message = (
+                f"Kubernetes job submission for data id {data.id} with args {argv} "
+                f" failed: '{error}'."
+            )
+            logger.exception(error_message)
+            data.process_error.append(error_message)
+            data.save()
 
         logger.debug(
             __(
@@ -773,29 +944,3 @@ class Connector(BaseConnector):
                 repr(argv),
             )
         )
-
-    def cleanup(self, data_id: int):
-        """Remove the persistent volume claims created by the executor."""
-        try:
-            kubernetes.config.load_incluster_config()
-        except kubernetes.config.config_exception.ConfigException:
-            kubernetes.config.load_kube_config()
-
-        core_api = kubernetes.client.CoreV1Api()
-        claim_names = [
-            unique_volume_name(type_, data_id)
-            for type_ in [
-                constants.PROCESSING_VOLUME_NAME,
-                constants.INPUTS_VOLUME_NAME,
-            ]
-            if type_ in storage_settings.FLOW_VOLUMES
-            and self._should_create_pvc(storage_settings.FLOW_VOLUMES[type_])
-        ]
-        for persistent_claim_name in claim_names:
-            logger.debug("Kubernetes: removing claim %s.", persistent_claim_name)
-            with suppress(kubernetes.client.rest.ApiException):
-                core_api.delete_namespaced_persistent_volume_claim(
-                    name=persistent_claim_name,
-                    namespace=self.kubernetes_namespace,
-                    _request_timeout=KUBERNETES_TIMEOUT,
-                )
