@@ -5,6 +5,7 @@ Slurm Connector
 ===============
 
 """
+
 import logging
 import os
 import shlex
@@ -34,7 +35,7 @@ class Connector(BaseConnector):
         For details, see
         :meth:`~resolwe.flow.managers.workload_connectors.base.BaseConnector.submit`.
         """
-        limits = data.process.get_resource_limits()
+        limits = data.get_resource_limits()
         logger.debug(
             __(
                 "Connector '{}' running for Data with id {} ({}).",
@@ -84,6 +85,3 @@ class Connector(BaseConnector):
                     err,
                 )
             )
-
-    def cleanup(self, data_id: int):
-        """Cleanup."""
