@@ -1,13 +1,20 @@
 """Resolwe collection model."""
+
+from django.contrib.contenttypes.models import ContentType
 from django.contrib.postgres.fields import ArrayField
 from django.contrib.postgres.indexes import GinIndex
 from django.contrib.postgres.search import SearchVectorField
-from django.db import models, transaction
+from django.db import models
 
+from resolwe.observers.consumers import BackgroundTaskType
+from resolwe.observers.models import BackgroundTask
+from resolwe.observers.utils import start_background_task
 from resolwe.permissions.models import PermissionObject, PermissionQuerySet
 
+from .annotations import AnnotationField
 from .base import BaseModel, BaseQuerySet
-from .utils import DirtyError, bulk_duplicate, validate_schema
+from .history_manager import HistoryMixin
+from .utils import DirtyError, validate_schema
 
 
 class BaseCollection(BaseModel):
@@ -52,19 +59,34 @@ class BaseCollection(BaseModel):
             raise ValueError(
                 "`descriptor_schema` must be defined if `descriptor` is given"
             )
-        super().save()
+        super().save(*args, **kwargs)
 
 
 class CollectionQuerySet(BaseQuerySet, PermissionQuerySet):
     """Query set for ``Collection`` objects."""
 
-    @transaction.atomic
-    def duplicate(self, contributor):
-        """Duplicate (make a copy) ``Collection`` objects."""
-        return bulk_duplicate(collections=self, contributor=contributor)
+    def duplicate(self, contributor) -> BackgroundTask:
+        """Duplicate (make a copy) ``Collection`` objects in the background."""
+        task_data = {"collection_ids": list(self.values_list("pk", flat=True))}
+        return start_background_task(
+            BackgroundTaskType.DUPLICATE_COLLECTION,
+            "Duplicate collections",
+            task_data,
+            contributor,
+        )
+
+    def delete_background(self, contributor):
+        """Delete the ``Collection`` objects in the background."""
+        task_data = {
+            "object_ids": list(self.values_list("pk", flat=True)),
+            "content_type_id": ContentType.objects.get_for_model(self.model).pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.DELETE, "Delete collections", task_data, contributor
+        )
 
 
-class Collection(BaseCollection, PermissionObject):
+class Collection(HistoryMixin, BaseCollection, PermissionObject):
     """Postgres model for storing a collection."""
 
     class Meta(BaseCollection.Meta):
@@ -97,16 +119,38 @@ class Collection(BaseCollection, PermissionObject):
 
     #: annotation fields available to samples in this collection
     annotation_fields = models.ManyToManyField(
-        "AnnotationField", related_name="collection"
+        AnnotationField, related_name="collection"
     )
 
     def is_duplicate(self):
         """Return True if collection is a duplicate."""
         return bool(self.duplicated)
 
-    def duplicate(self, contributor):
-        """Duplicate (make a copy)."""
-        return bulk_duplicate(
-            collections=self._meta.model.objects.filter(pk=self.pk),
-            contributor=contributor,
-        )[0]
+    def duplicate(self, contributor) -> BackgroundTask:
+        """Duplicate (make a copy) object in the background."""
+        task_data = {"collection_ids": [self.pk]}
+        return start_background_task(
+            BackgroundTaskType.DUPLICATE_COLLECTION,
+            "Duplicate collection",
+            task_data,
+            contributor,
+        )
+
+    def delete_background(self):
+        """Delete the object in the background."""
+        task_data = {
+            "object_ids": [self.pk],
+            "content_type_id": ContentType.objects.get_for_model(self).pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.DELETE, "Delete collections", task_data, self.contributor
+        )
+
+    def save(self, *args, **kwargs):
+        """Add required annotation fields to the collection."""
+        create = self.pk is None
+        super().save(*args, **kwargs)
+        if create:
+            required_fields = AnnotationField.objects.filter(required=True)
+            if required_fields.exists():
+                self.annotation_fields.add(*required_fields)
