@@ -1,11 +1,14 @@
 """Reslowe process model."""
+
 import copy
 import enum
 import json
 import logging
+from typing import Any, Optional, Union
 
 import jsonschema
 
+from django.contrib.contenttypes.models import ContentType
 from django.contrib.postgres.fields import ArrayField
 from django.contrib.postgres.indexes import GinIndex
 from django.contrib.postgres.search import SearchVectorField
@@ -21,17 +24,20 @@ from resolwe.flow.models.utils import (
     validation_schema,
 )
 from resolwe.flow.utils import dict_dot, get_data_checksum, iterate_fields
-from resolwe.observers.protocol import post_permission_changed, pre_permission_changed
+from resolwe.observers.consumers import BackgroundTaskType
+from resolwe.observers.decorators import move_to_container
+from resolwe.observers.models import BackgroundTask
+from resolwe.observers.utils import start_background_task
 from resolwe.permissions.models import PermissionObject, PermissionQuerySet
 from resolwe.permissions.utils import assign_contributor_permissions, copy_permissions
 
 from .base import BaseModel, BaseQuerySet
 from .descriptor import DescriptorSchema
 from .entity import Entity, EntityQuerySet
+from .history_manager import HistoryMixin
 from .secret import Secret
 from .storage import Storage
 from .utils import (
-    bulk_duplicate,
     hydrate_input_references,
     hydrate_size,
     render_descriptor,
@@ -59,7 +65,7 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
     """Query set for Data objects."""
 
     @staticmethod
-    def _handle_entity(obj):
+    def _handle_entity(obj: "Data"):
         """Create entity if `entity.type` is defined in process.
 
         Following rules applies for adding `Data` object to `Entity`:
@@ -79,7 +85,7 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
         operation = HandleEntityOperation.PASS
 
         if entity_type:
-            data_filter = {}
+            data_filter: dict[str, Union[list, int]] = {}
             if entity_input:
                 input_id = dict_dot(obj.input, entity_input, default=lambda: None)
                 if input_id is None:
@@ -115,28 +121,23 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
                     tags=obj.tags,
                 )
                 assign_contributor_permissions(entity)
+                obj.move_to_entity(entity)
                 operation = HandleEntityOperation.CREATE
 
             elif entity_count == 1:
                 entity = entity_query.first()
-                obj.tags = entity.tags
-                copy_permissions(entity, obj)
                 operation = HandleEntityOperation.ADD
+                obj.move_to_entity(entity)
 
             else:
                 logger.info(
                     "Skipping creation of entity due to multiple entities found."
                 )
-                entity = None
-
-            if entity:
-                obj.entity = entity
-                obj.save()
 
             return operation
 
     @staticmethod
-    def _handle_collection(obj, entity_operation=None):
+    def _handle_collection(obj: "Data", entity_operation: HandleEntityOperation):
         """Correctly assign Collection to Data and it's Entity.
 
         There are 2 x 4 possible scenarios how to handle collection
@@ -169,15 +170,10 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
 
         # 1.4
         if not obj.collection and obj.entity and obj.entity.collection:
-            obj.collection = obj.entity.collection
-            obj.tags = obj.entity.collection.tags
-            obj.save()
+            obj.move_to_collection(obj.entity.collection)
         # 2.2
         if entity_operation == HandleEntityOperation.CREATE and obj.collection:
-            obj.entity.collection = obj.collection
-            obj.entity.tags = obj.collection.tags
-            obj.entity.save()
-            copy_permissions(obj.collection, obj.entity)
+            obj.entity.move_to_collection(obj.collection)
 
     @transaction.atomic
     def create(self, subprocess_parent=None, **kwargs):
@@ -206,18 +202,25 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
 
         return obj
 
-    @transaction.atomic
-    def duplicate(self, contributor, inherit_entity=False, inherit_collection=False):
-        """Duplicate (make a copy) ``Data`` objects.
+    def duplicate(self, contributor) -> BackgroundTask:
+        """Duplicate (make a copy) ``Data`` objects in the background."""
+        task_data = {
+            "data_ids": list(self.values_list("pk", flat=True)),
+            "inherit_entity": True,
+            "inherit_collection": True,
+        }
+        return start_background_task(
+            BackgroundTaskType.DUPLICATE_DATA, "Duplicate data", task_data, contributor
+        )
 
-        :param contributor: Duplication user
-        """
-
-        return bulk_duplicate(
-            data=self,
-            contributor=contributor,
-            inherit_entity=inherit_entity,
-            inherit_collection=inherit_collection,
+    def delete_background(self, contributor):
+        """Delete the ``Data`` objects in the background."""
+        task_data = {
+            "object_ids": list(self.values_list("pk", flat=True)),
+            "content_type_id": ContentType.objects.get_for_model(self.model).pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.DELETE, "Delete data", task_data, contributor
         )
 
     @transaction.atomic
@@ -252,7 +255,7 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
         return self.annotate(**annotation_data)
 
 
-class Data(BaseModel, PermissionObject):
+class Data(HistoryMixin, BaseModel, PermissionObject):
     """Postgres model for storing data."""
 
     class Meta(BaseModel.Meta):
@@ -582,41 +585,54 @@ class Data(BaseModel, PermissionObject):
 
         Storage.objects.filter(pk__in=storage_ids, data=None).delete()
 
+    def delete_background(self):
+        """Delete the object in the background."""
+        task_data = {
+            "object_ids": [self.pk],
+            "content_type_id": ContentType.objects.get_for_model(self).pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.DELETE, "Delete data", task_data, self.contributor
+        )
+
     def is_duplicate(self):
         """Return True if data object is a duplicate."""
         return bool(self.duplicated)
 
-    def duplicate(self, contributor, inherit_entity=False, inherit_collection=False):
-        """Duplicate (make a copy)."""
-        return bulk_duplicate(
-            data=self._meta.model.objects.filter(pk=self.pk),
-            contributor=contributor,
-            inherit_entity=inherit_entity,
-            inherit_collection=inherit_collection,
-        )[0]
+    def duplicate(self, contributor) -> BackgroundTask:
+        """Duplicate (make a copy) object in the background."""
+        task_data = {
+            "data_ids": [self.pk],
+            "inherit_entity": True,
+            "inherit_collection": True,
+        }
+        return start_background_task(
+            BackgroundTaskType.DUPLICATE_DATA, "Duplicate data", task_data, contributor
+        )
 
+    @move_to_container
     def move_to_collection(self, collection):
         """Move data object to collection."""
         self.validate_change_collection(collection)
-        pre_permission_changed.send(sender=type(self), instance=self)
         self.collection = collection
         self.permission_group = collection.permission_group
         if collection:
             self.tags = collection.tags
         self.save(update_fields=["tags", "permission_group", "collection"])
-        post_permission_changed.send(sender=type(self), instance=self)
 
+    @move_to_container
     def move_to_entity(self, entity):
         """Move data object to entity."""
         if entity is None and self.in_container():
             raise ValidationError("Data object must not be removed from container.")
-        pre_permission_changed.send(sender=type(self), instance=self)
         self.entity = entity
         if entity:
-            self.permission_group = entity.permission_group
-        self.tags = entity.tags
+            if entity.collection:
+                self.move_to_collection(entity.collection)
+            else:
+                self.permission_group = entity.permission_group
+                self.tags = entity.tags
         self.save(update_fields=["permission_group", "entity", "tags"])
-        post_permission_changed.send(sender=type(self), instance=self)
 
     def validate_change_collection(self, collection):
         """Raise validation error if data object cannot change collection."""
@@ -657,6 +673,140 @@ class Data(BaseModel, PermissionObject):
         """Return the resource limits for this data."""
         return self.process.get_resource_limits(self)
 
+    def restart(self, resource_overrides: dict = {}):
+        """Restart the data object and all its children.
+
+        The status of the data object must be ERROR and stasus of its dependencies must
+        be DONE.
+
+        :param resource_overrides: dictionary mapping ids of data objects to resource
+            overrides.
+
+        :raises RuntimeError: if the object is not in the right state.
+        :raises RuntimeError: when object dependencies are not in the status DONE.
+        """
+
+        def get_dependencies(data: Data) -> set[Data]:
+            """Get the dependencies of the data object.
+
+            Also include the data object itself.
+            """
+            dependencies = set()
+            stack = [data]
+            while stack:
+                if (processing := stack.pop()) not in dependencies:
+                    stack += Data.objects.filter(
+                        parents_dependency__parent=processing,
+                        parents_dependency__kind__in=[DataDependency.KIND_IO],
+                    )
+                    dependencies.add(processing)
+            return dependencies
+
+        def reset_data(data: Data):
+            """Prepare the data object for another processing."""
+            assert data.status == Data.STATUS_ERROR
+            reset_dict: dict[str, Any] = {
+                "started": None,
+                "finished": None,
+                "status": Data.STATUS_RESOLVING,
+                "process_info": [],
+                "process_warning": [],
+                "process_error": [],
+                "size": 0,
+                "process_progress": 0,
+                "process_rc": None,
+                "process_pid": None,
+                "descriptor": {},
+                "descriptor_dirty": False,
+                "duplicated": None,
+            }
+            for attribute, value in reset_dict.items():
+                setattr(data, attribute, value)
+            if hasattr(data, "worker"):
+                # The worker can be saved here as it is not observable.
+                data.worker.status = Worker.STATUS_PREPARING
+                data.worker.save()
+
+        # Avoid circular dependencies.
+        from resolwe.flow.signals import commit_signal
+
+        # Sanity check: only data in status ERROR can be restarted.
+        if self.status != Data.STATUS_ERROR:
+            raise RuntimeError(
+                f"Only data in status {Data.STATUS_ERROR} can be restarted, not {self.status}."
+            )
+
+        # Sanity check: all dependencies must be in status DONE.
+        if self.dependency_status() != Data.STATUS_DONE:
+            raise RuntimeError(
+                f"Data dependencies must have status '{Data.STATUS_DONE}'."
+            )
+
+        with transaction.atomic():
+            # When data object is restarted we also need to restart all its children.
+            # We have to clear them all and reset their statuses to be restarted.
+            dependencies = get_dependencies(self)
+
+            # Subprocess dependencies must be deleted, they will be recreated.
+            Data.objects.filter(
+                parents_dependency__parent=self,
+                parents_dependency__kind__in=[DataDependency.KIND_SUBPROCESS],
+            ).delete()
+
+            # Construct the map id -> data.
+            to_process = {data.id: data for data in dependencies}
+
+            # Prevent circular import.
+            from resolwe.flow.managers.listener.listener import cache_manager
+
+            # Clear the Redis cache for objects to be restarted.
+            for data_id in to_process.keys():
+                cache_manager.clear(Data, (data_id,))
+
+            # Evaluate lazy generator by listing it.
+            list(map(reset_data, to_process.values()))
+
+            # Set the overrides.
+            for data_pk, override in resource_overrides.items():
+                if int(data_pk) in to_process:
+                    to_process[int(data_pk)].process_resources = override
+
+            # Save the data objects. This will send out notifications.
+            for data in to_process.values():
+                data.save()
+
+        # Start processing the duplicate.
+        commit_signal(self, False, update_fields=None)
+
+    def dependency_status(self) -> Optional[str]:
+        """Return abstracted status of instance IO dependencies.
+
+        :returns:
+            - ``STATUS_ERROR`` .. one dependency has error status or was deleted
+            - ``STATUS_DONE`` .. all dependencies have done status
+            - ``None`` .. other
+        """
+        parents_statuses = set(
+            DataDependency.objects.filter(child=self, kind=DataDependency.KIND_IO)
+            .distinct("parent__status")
+            .values_list("parent__status", flat=True)
+        )
+
+        if not parents_statuses:
+            return Data.STATUS_DONE
+
+        if None in parents_statuses:
+            # Some parents have been deleted.
+            return Data.STATUS_ERROR
+
+        if Data.STATUS_ERROR in parents_statuses:
+            return Data.STATUS_ERROR
+
+        if len(parents_statuses) == 1 and Data.STATUS_DONE in parents_statuses:
+            return Data.STATUS_DONE
+
+        return None
+
 
 class DataDependency(models.Model):
     """Dependency relation between data objects."""
