@@ -1,56 +1,55 @@
 """Resolwe entity model."""
+
 from typing import Any, List, Optional, Union
 
-from django.contrib.postgres.fields import CICharField
+from django.contrib.contenttypes.models import ContentType
 from django.contrib.postgres.indexes import GinIndex
 from django.core.exceptions import ValidationError
 from django.db import models, transaction
 
-from resolwe.flow.models.annotations import (
-    AnnotationField,
-    AnnotationValue,
-    HandleMissingAnnotations,
-)
-from resolwe.observers.protocol import post_permission_changed, pre_permission_changed
+from resolwe.flow.models.annotations import AnnotationField, AnnotationValue
+from resolwe.observers.consumers import BackgroundTaskType
+from resolwe.observers.decorators import move_to_container
+from resolwe.observers.models import BackgroundTask
+from resolwe.observers.utils import start_background_task
 from resolwe.permissions.models import PermissionObject, PermissionQuerySet
 
 from .base import BaseModel, BaseQuerySet
 from .collection import BaseCollection, Collection
-from .utils import DirtyError, bulk_duplicate, validate_schema
+from .utils import DirtyError, validate_schema
 
 
 class EntityQuerySet(BaseQuerySet, PermissionQuerySet):
     """Query set for ``Entity`` objects."""
 
-    @transaction.atomic
-    def duplicate(
-        self, contributor, inherit_collection: bool = False
-    ) -> models.QuerySet:
-        """Duplicate (make a copy) ``Entity`` objects.
+    def duplicate(self, contributor) -> BackgroundTask:
+        """Duplicate (make a copy) ``Entity`` objects in the background."""
+        task_data = {
+            "entity_ids": list(self.values_list("pk", flat=True)),
+            "inherit_collection": True,
+        }
+        return start_background_task(
+            BackgroundTaskType.DUPLICATE_ENTITY,
+            "Duplicate entity",
+            task_data,
+            contributor,
+        )
 
-        :param contributor: Duplication user
-        :param inherit_collection: If ``True`` then duplicated
-            entities will be added to collection the original entity
-            is part of. Duplicated entities' data objects will also be
-            added to the collection, but only those which are in the
-            collection
-        :return: A list of duplicated entities
-        """
-        return bulk_duplicate(
-            entities=self,
-            contributor=contributor,
-            inherit_collection=inherit_collection,
+    def delete_background(self, contributor):
+        """Delete the ``Entity`` objects in the background."""
+        task_data = {
+            "object_ids": list(self.values_list("pk", flat=True)),
+            "content_type_id": ContentType.objects.get_for_model(self.model).pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.DELETE, "Delete entities", task_data, contributor
         )
 
     @transaction.atomic
-    def move_to_collection(
-        self,
-        destination_collection: Collection,
-        missing_annotations: HandleMissingAnnotations = HandleMissingAnnotations.ADD,
-    ):
+    def move_to_collection(self, destination_collection: Collection):
         """Move entities to destination collection."""
         for entity in self:
-            entity.move_to_collection(destination_collection, missing_annotations)
+            entity.move_to_collection(destination_collection)
 
     def annotate_all(self, add_labels: bool = False):
         """Annotate with all metadata on the entities.
@@ -225,25 +224,32 @@ class Entity(BaseCollection, PermissionObject):
         """Return True if entity is a duplicate."""
         return bool(self.duplicated)
 
-    def duplicate(self, contributor, inherit_collection: bool = False) -> "Entity":
-        """Duplicate (make a copy)."""
-        return bulk_duplicate(
-            entities=self._meta.model.objects.filter(pk=self.pk),
-            contributor=contributor,
-            inherit_collection=inherit_collection,
-        )[0]
+    def duplicate(self, contributor) -> BackgroundTask:
+        """Duplicate (make a copy) object in the background."""
+        task_data = {"entity_ids": [self.pk], "inherit_collection": True}
+        return start_background_task(
+            BackgroundTaskType.DUPLICATE_ENTITY,
+            "Duplicate entity",
+            task_data,
+            contributor,
+        )
+
+    def delete_background(self):
+        """Delete the object in the background."""
+        task_data = {
+            "object_ids": [self.pk],
+            "content_type_id": ContentType.objects.get_for_model(self).pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.DELETE, "Delete entitiy", task_data, self.contributor
+        )
 
     @transaction.atomic
-    def move_to_collection(
-        self,
-        collection: Collection,
-        handle_missing_annotations: HandleMissingAnnotations = HandleMissingAnnotations.ADD,
-    ):
+    @move_to_container
+    def move_to_collection(self, collection: Collection):
         """Move entity from the source to the destination collection.
 
         :args collection: the collection to move entity into.
-        :args handle_missing_annotations: how to handle missing annotations fields in
-            the new collection
 
         :raises ValidationError: when collection is set to None.
         """
@@ -254,25 +260,18 @@ class Entity(BaseCollection, PermissionObject):
             raise ValidationError(
                 f"Entity {self}({self.pk}) can only be moved to another container."
             )
-        pre_permission_changed.send(sender=type(self), instance=self)
 
-        missing_annotations = AnnotationField.objects.none()
-        if self.collection is not None:
-            missing_annotations = self.collection.annotation_fields.exclude(
+        if self.collection and (
+            missing_annotations := self.collection.annotation_fields.exclude(
                 pk__in=collection.annotation_fields.values_list("pk", flat=True)
             )
-
-        if missing_annotations:
-            if handle_missing_annotations == HandleMissingAnnotations.ADD:
-                collection.annotation_fields.add(*missing_annotations)
-            elif handle_missing_annotations == HandleMissingAnnotations.REMOVE:
-                self.annotations.filter(field__in=missing_annotations).delete()
+        ):
+            collection.annotation_fields.add(*missing_annotations)
 
         self.collection = collection
         self.tags = collection.tags
         self.permission_group = collection.permission_group
         self.save(update_fields=["collection", "tags", "permission_group"])
-        post_permission_changed.send(sender=type(self), instance=self)
         self.data.move_to_collection(collection)
 
     def invalid_annotation_fields(self, annotation_fields=None):
@@ -293,8 +292,7 @@ class Entity(BaseCollection, PermissionObject):
     def copy_annotations(self, destination: "Entity") -> List[AnnotationValue]:
         """Copy annotation from this entity to the destination.
 
-        :raises ValidationError: when some of the annotation fields are missing on the
-            destination entity.
+        The objects are not saved to the database.
         """
         # Entity without collection can not have annotations.
         if self.collection is None:
@@ -310,27 +308,12 @@ class Entity(BaseCollection, PermissionObject):
         # destination entity, else exception is raised.
         for source_annotation in self.annotations.all():
             annotation_value = AnnotationValue(
-                entity=destination, field_id=source_annotation.field_id
+                entity=destination,
+                field_id=source_annotation.field_id,
+                value=source_annotation.value,
             )
-            annotation_value.value = source_annotation.value
             destination_annotations.append(annotation_value)
             annotation_field_ids.add(source_annotation.field_id)
-
-        destination_fields_qs = destination.collection.annotation_fields.filter(
-            pk__in=annotation_field_ids
-        )
-        if destination_fields_qs.count() < len(annotation_field_ids):
-            missing_annotation_field_ids = annotation_field_ids - set(
-                destination_fields_qs.values_list("pk", flat=True)
-            )
-            missing_fields_qs = AnnotationField.objects.filter(
-                pk__in=missing_annotation_field_ids
-            ).values_list("group__name", "name")
-            missing_field_paths = [".".join(entry) for entry in missing_fields_qs]
-            missing_fields = ", ".join(missing_field_paths)
-            raise ValidationError(
-                f"The collection of entity '{destination}' is missing annotation fields {missing_fields}."
-            )
         return destination_annotations
 
     def validate_annotations(self):
@@ -359,6 +342,57 @@ class Entity(BaseCollection, PermissionObject):
         if validation_errors:
             raise ValidationError(validation_errors)
 
+    def update_annotations(self, annotations: dict[str, Any], update=True):
+        """Update annotations with the given values.
+
+        When annotation value is set no None it is deleted.
+
+        :attr annotations: the dictionary with annotation values. Keys are annotation
+            paths.
+        """
+        field_paths = annotations.keys()
+        field_map = {
+            field_path: AnnotationField.field_from_path(field_path)
+            for field_path in field_paths
+        }
+
+        # Create annotation values and prepare list of fields which annotations should
+        # be deleted.
+        annotation_values: list[AnnotationValue] = []
+        validation_errors: list[ValidationError] = []
+        fields_to_delete: list[int] = []
+        for field_path, field_value in annotations.items():
+            if field_value is None:
+                fields_to_delete.append(field_map[field_path].pk)
+            else:
+                value = AnnotationValue(
+                    field=field_map[field_path], value=field_value, entity=self
+                )
+                annotation_values.append(value)
+                try:
+                    value.validate()
+                except ValidationError as e:
+                    validation_errors.append(e)
+        if validation_errors:
+            raise ValidationError(validation_errors)
+
+        # Delete and update annotations in a transaction.
+        with transaction.atomic():
+            to_delete = AnnotationValue.objects.filter(entity=self)
+            if update:
+                to_delete = to_delete.filter(field_id__in=fields_to_delete)
+            to_delete.delete()
+            AnnotationValue.objects.bulk_create(
+                annotation_values,
+                update_conflicts=True,
+                update_fields=["_value"],
+                unique_fields=["entity", "field"],
+            )
+
+        # Add missing annotation fields to the collection.
+        if self.collection is not None:
+            self.collection.annotation_fields.add(*field_map.values())
+
 
 class RelationType(models.Model):
     """Model for storing relation types."""
@@ -439,7 +473,7 @@ class Relation(BaseModel, PermissionObject):
     entities = models.ManyToManyField(Entity, through="RelationPartition")
 
     #: category of the relation
-    category = CICharField(max_length=100)
+    category = models.CharField(max_length=100)
 
     #: unit used in the partitions' positions (where applicable, e.g. for serieses)
     unit = models.CharField(max_length=3, choices=UNIT_CHOICES, null=True, blank=True)
