@@ -1,8 +1,10 @@
 """Resolwe storage model."""
-from django.contrib.postgres.fields import JSONField
+
 from django.db import models
 
-from .base import BaseModel, BaseQuerySet
+from resolwe.permissions.models import PermissionQuerySet
+
+from .base import BaseModel
 from .functions import JsonGetPath
 from .utils import json_path_components
 
@@ -43,10 +45,10 @@ class Storage(BaseModel):
     data = models.ManyToManyField("Data", related_name="storages")
 
     #: actual JSON stored
-    json = JSONField()
+    json = models.JSONField()
 
     #: storage manager
-    objects = StorageManager.from_queryset(BaseQuerySet)()
+    objects = StorageManager.from_queryset(PermissionQuerySet)()
 
 
 class LazyStorageJSON:
