@@ -1,4 +1,7 @@
 """Resolwe data serializer."""
+
+from django.db import transaction
+
 from rest_framework import serializers
 
 from resolwe.flow.models import Collection, Data, DescriptorSchema, Entity, Process
@@ -8,7 +11,7 @@ from resolwe.rest.fields import ProjectableJSONField
 from .base import ResolweBaseSerializer
 from .collection import CollectionSerializer
 from .descriptor import DescriptorSchemaSerializer
-from .entity import EntitySerializer
+from .entity import BaseEntitySerializer
 from .fields import DictRelatedField
 from .process import ProcessSerializer
 
@@ -38,7 +41,7 @@ class DataSerializer(ResolweBaseSerializer):
     )
     entity = DictRelatedField(
         queryset=Entity.objects.all(),
-        serializer=EntitySerializer,
+        serializer=BaseEntitySerializer,
         allow_null=True,
         required=False,
         write_permission=Permission.EDIT,
@@ -103,11 +106,12 @@ class DataSerializer(ResolweBaseSerializer):
             self.instance.validate_change_collection(collection)
         return collection
 
+    @transaction.atomic
     def update(self, instance, validated_data):
         """Update."""
-        pre_update_collection = instance.collection
+        update_collection = "collection" in validated_data
+        new_collection = validated_data.pop("collection", None)
         instance = super().update(instance, validated_data)
-        if pre_update_collection != instance.collection:
-            instance.move_to_collection(instance.collection)
-
+        if update_collection and new_collection != instance.collection:
+            instance.move_to_collection(new_collection)
         return instance
