@@ -1,15 +1,42 @@
 """Resolwe custom serializer fields."""
+
 from collections import OrderedDict
 
+from drf_spectacular.utils import extend_schema_field
+
 from django.core.exceptions import ObjectDoesNotExist
-from django.utils.encoding import smart_text
+from django.utils.encoding import smart_str
 
-from rest_framework import exceptions, relations
+from rest_framework import exceptions, relations, serializers
 
-from resolwe.permissions.shortcuts import get_objects_for_user
-from resolwe.permissions.utils import get_full_perm
+from resolwe.permissions.models import Permission
 
 
+class PrimaryKeyDictRelatedField(serializers.PrimaryKeyRelatedField):
+    """Deserialize/serialize model to the dict with 'id' key."""
+
+    def to_internal_value(self, data):
+        """Extract id from dict and use the parent method."""
+        try:
+            if not isinstance(data, dict) or "id" not in data:
+                raise TypeError
+            return super().to_internal_value(data["id"])
+        except (TypeError, ValueError):
+            self.fail("incorrect_type", data_type=type(data).__name__)
+
+    def to_representation(self, value):
+        """Convert to dict representation - only id gets serialized."""
+        return {"id": value.pk}
+
+
+class RelationshipDictSerializer(serializers.Serializer):
+    """Serializer for the DictRelatedField relation."""
+
+    id = serializers.IntegerField(required=False)
+    slug = serializers.CharField(required=False, allow_null=True)
+
+
+@extend_schema_field(RelationshipDictSerializer)
 class DictRelatedField(relations.RelatedField):
     """
     Field representing the target of the relationship by using dict.
@@ -34,7 +61,7 @@ class DictRelatedField(relations.RelatedField):
         ),
     }
 
-    def __init__(self, serializer, write_permission="view", **kwargs):
+    def __init__(self, serializer, write_permission=Permission.VIEW, **kwargs):
         """Initialize attributes."""
         self.serializer = serializer
         self.write_permission = write_permission
@@ -63,18 +90,20 @@ class DictRelatedField(relations.RelatedField):
 
         user = getattr(self.context.get("request"), "user")
         queryset = self.get_queryset()
-        permission = get_full_perm(self.write_permission, queryset.model)
+        permission = self.write_permission
         try:
-            return get_objects_for_user(
-                user, permission, queryset.filter(**kwargs)
-            ).latest("version")
+            return (
+                queryset.filter(**kwargs)
+                .filter_for_user(user, permission)
+                .latest("version")
+            )
         except ObjectDoesNotExist:
             # Differentiate between "user has no permission" and "object does not exist"
-            view_permission = get_full_perm("view", queryset.model)
+            view_permission = Permission.VIEW
             if permission != view_permission:
                 try:
-                    get_objects_for_user(
-                        user, view_permission, queryset.filter(**kwargs)
+                    queryset.filter(**kwargs).filter_for_user(
+                        user, view_permission
                     ).latest("version")
                     raise exceptions.PermissionDenied(
                         "You do not have {} permission for {}: {}.".format(
@@ -85,7 +114,7 @@ class DictRelatedField(relations.RelatedField):
                     pass
 
             self.fail(
-                "does_not_exist", value=smart_text(data), model_name=self.model_name
+                "does_not_exist", value=smart_str(data), model_name=self.model_name
             )
 
     def to_representation(self, obj):
