@@ -1,4 +1,5 @@
 """Resolwe relation serializer."""
+
 from django.db import transaction
 
 from rest_framework import serializers
@@ -66,6 +67,7 @@ class RelationSerializer(ResolweBaseSerializer):
                 "collection",
                 "category",
                 "partitions",
+                "slug",
                 "unit",
                 "descriptor",
                 "descriptor_schema",
@@ -89,12 +91,21 @@ class RelationSerializer(ResolweBaseSerializer):
                 position=partition.get("position", None),
             )
 
+    def _make_category_lower_case(self, validated_data: dict):
+        """Return the category name in lower case.
+
+        This avoids creating duplicate categories with different case.
+        """
+        if "category" in validated_data:
+            validated_data["category"] = validated_data["category"].lower()
+
     def create(self, validated_data):
         """Create ``Relation`` object and add partitions of ``Entities``."""
         # `partitions` field is renamed to `relationpartition_set` based on source of nested serializer
         partitions = validated_data.pop("relationpartition_set")
 
         with transaction.atomic():
+            self._make_category_lower_case(validated_data)
             instance = Relation.objects.create(**validated_data)
             self._create_partitions(instance, partitions)
 
@@ -106,6 +117,7 @@ class RelationSerializer(ResolweBaseSerializer):
         partitions = validated_data.pop("relationpartition_set", None)
 
         with transaction.atomic():
+            self._make_category_lower_case(validated_data)
             instance = super().update(instance, validated_data)
 
             if partitions is not None:
