@@ -4,29 +4,36 @@ import logging
 import os
 import subprocess
 import sys
-import threading
 import unittest
-from contextlib import suppress
+from concurrent.futures import ThreadPoolExecutor
 from pathlib import Path
-from time import sleep
+from time import sleep, time
 from unittest import mock
+from unittest.mock import patch
 
 import zmq
 import zmq.asyncio
-from asgiref.sync import async_to_sync
 
 from django.conf import settings
+from django.db import IntegrityError, connection
 from django.test import override_settings
 
-from guardian.shortcuts import assign_perm
-
 from resolwe.flow.executors.prepare import BaseFlowExecutorPreparer
 from resolwe.flow.executors.socket_utils import Message
-from resolwe.flow.executors.startup_processing_container import ProcessingManager
+from resolwe.flow.executors.startup_processing_container import (
+    connect_to_communication_container,
+    initialize_connections,
+)
 from resolwe.flow.executors.zeromq_utils import ZMQCommunicator
-from resolwe.flow.managers import manager
-from resolwe.flow.managers.utils import disable_auto_calls
+from resolwe.flow.managers.dispatcher import Manager
+from resolwe.flow.managers.listener.redis_cache import redis_cache
 from resolwe.flow.models import Data, DataDependency, Process, Worker
+from resolwe.flow.models.annotations import (
+    AnnotationField,
+    AnnotationGroup,
+    AnnotationType,
+)
+from resolwe.flow.models.fields import ResolweSlugField
 from resolwe.storage import settings as storage_settings
 from resolwe.test import (
     ProcessTestCase,
@@ -39,6 +46,8 @@ from resolwe.test import (
 PROCESSES_DIR = os.path.join(os.path.dirname(__file__), "processes")
 DESCRIPTORS_DIR = os.path.join(os.path.dirname(__file__), "descriptors")
 
+logger = logging.getLogger(__name__)
+
 
 class GetToolsTestCase(TestCase):
     @mock.patch("resolwe.flow.utils.apps")
@@ -90,31 +99,178 @@ class ManagerRunProcessTest(ProcessTestCase):
         self._register_schemas(
             processes_paths=[PROCESSES_DIR], descriptors_paths=[DESCRIPTORS_DIR]
         )
-
-    @with_docker_executor
-    @tag_process("test-min-python34")
-    def test_python_34(self):
-        """Test that processing container starts on Python 3.4.
-
-        This test can be removed when we stop using
-        broadinstitute/genomes-in-the-cloud:2.3.1-1504795437 docker image in
-        the GATK3 pipeline.
-        """
-        data = self.run_process("test-min-python34")
-        self.assertEqual(data.output["out"], "OK")
+        group = AnnotationGroup.objects.create(name="general", sort_order=1)
+        self.species_field = AnnotationField.objects.create(
+            name="species",
+            group=group,
+            sort_order=1,
+            type=AnnotationType.STRING.value,
+            vocabulary={"Valid": "Valid"},
+        )
+        AnnotationField.objects.create(
+            name="description",
+            group=group,
+            sort_order=2,
+            type=AnnotationType.STRING.value,
+        )
 
     @tag_process("test-min")
     def test_minimal_process(self):
         self.run_process("test-min")
 
+    @tag_process("test-min")
+    @tag_process("test-workflow-simple")
+    def test_restart(self):
+        def restarted_object_sanity_check(
+            original_values, restarted_values, additional_skip_keys=[]
+        ):
+            self.assertEqual(restarted_values["status"], Data.STATUS_DONE)
+            self.assertEqual(original_values.keys(), restarted_values.keys())
+
+            # These values should be both None or modified greater than original.
+            for key in {"started", "finished", "modified", "scheduled"}:
+                if restarted_values[key] is not None:
+                    compare_value = original_values["finished"] or original_values[key]
+                    self.assertGreater(restarted_values[key], compare_value)
+                else:
+                    self.assertEqual(restarted_values[key], original_values[key])
+
+            skip_keys = {"started", "finished", "modified", "scheduled", "location_id"}
+            skip_keys.update(additional_skip_keys)
+            for key in original_values.keys():
+                if key not in skip_keys:
+                    self.assertEqual(
+                        original_values[key],
+                        restarted_values[key],
+                        f"Key '{key}' does not match.",
+                    )
+
+        original = self.run_process("test-min", contributor=self.contributor)
+        with self.assertRaises(
+            RuntimeError, msg="Only data in status ERROR can be restarted, not OK."
+        ):
+            original.restart()
+
+        original_values = Data.objects.filter(pk=original.pk).values()[0]
+
+        # No need to actually save the status, it is checked on the instance.
+        original.status = Data.STATUS_ERROR
+        original.restart()
+
+        restarted_values = Data.objects.filter(pk=original.pk).values()[0]
+        restarted_object_sanity_check(original_values, restarted_values)
+
+        # Now test the workflow duplication. The workflow creates two data objects
+        # and the second one depends on the first one.
+        workflow_data = self.run_process(
+            "test-workflow-simple",
+            {"param1": "world"},
+            tags=["test-tag"],
+            contributor=self.contributor,
+        )
+        workflow_data.refresh_from_db()
+        step1_data = Data.objects.get(process__slug="test-example-1-simple")
+        step2_data = Data.objects.get(process__slug="test-example-2-simple")
+
+        workflow_dict = Data.objects.filter(pk=workflow_data.pk).values()[0]
+        step1_dict = Data.objects.filter(pk=step1_data.pk).values()[0]
+        step2_dict = Data.objects.filter(pk=step2_data.pk).values()[0]
+
+        # First try to duplicate the entire workflow.
+        workflow_data.status = Data.STATUS_ERROR
+        workflow_data.restart()
+
+        # Subprocess dependencies must be deleted.
+        with self.assertRaises(Data.DoesNotExist):
+            step1_data.refresh_from_db()
+        with self.assertRaises(Data.DoesNotExist):
+            step2_data.refresh_from_db()
+
+        restarted_workflow_dict = Data.objects.filter(pk=workflow_data.pk).values()[0]
+        restarted_step1_dict = Data.objects.filter(
+            process__slug="test-example-1-simple"
+        ).values()[0]
+        restarted_step2_dict = Data.objects.filter(
+            process__slug="test-example-2-simple"
+        ).values()[0]
+        # Output also differs (different step ids).
+        restarted_object_sanity_check(
+            workflow_dict, restarted_workflow_dict, ["output"]
+        )
+        # Ids will not match, since object is restarted.
+        self.assertNotEqual(restarted_step1_dict["id"], step1_dict["id"])
+        self.assertNotEqual(restarted_step2_dict["id"], step2_dict["id"])
+        self.assertGreater(restarted_step1_dict["created"], step1_dict["created"])
+        self.assertGreater(restarted_step2_dict["created"], step2_dict["created"])
+        restarted_object_sanity_check(
+            step1_dict, restarted_step1_dict, ["id", "permission_group_id", "created"]
+        )
+        restarted_object_sanity_check(
+            step2_dict, restarted_step2_dict, ["id", "permission_group_id", "created"]
+        )
+
+        # Override resources on step1 and step2. Be careful: when restarting the entire
+        # workflow the workflow will spawn new objects and its children will be restarted.
+        Data.objects.all().delete()
+        workflow_data = self.run_process(
+            "test-workflow-simple",
+            {"param1": "world"},
+            tags=["test-tag"],
+            contributor=self.contributor,
+        )
+        workflow_data.refresh_from_db()
+
+        step1_data = Data.objects.get(process__slug="test-example-1-simple")
+        step2_data = Data.objects.get(process__slug="test-example-2-simple")
+
+        workflow_dict = Data.objects.filter(pk=workflow_data.pk).values()[0]
+        step1_dict = Data.objects.filter(pk=step1_data.pk).values()[0]
+        step2_dict = Data.objects.filter(pk=step2_data.pk).values()[0]
+
+        # Duplicate the entire workflow, override resources.
+        workflow_data.status = Data.STATUS_ERROR
+
+        resource_overrides = {}
+        resource_overrides[workflow_data.pk] = {"cores": 1}
+        workflow_data.restart(resource_overrides=resource_overrides)
+
+        # Subprocess dependencies must be deleted.
+        with self.assertRaises(Data.DoesNotExist):
+            step1_data.refresh_from_db()
+        with self.assertRaises(Data.DoesNotExist):
+            step2_data.refresh_from_db()
+
+        restarted_workflow_dict = Data.objects.filter(pk=workflow_data.pk).values()[0]
+        restarted_step1_dict = Data.objects.filter(
+            process__slug="test-example-1-simple"
+        ).values()[0]
+        restarted_step2_dict = Data.objects.filter(
+            process__slug="test-example-2-simple"
+        ).values()[0]
+        # Output also differs (different step ids).
+        restarted_object_sanity_check(
+            workflow_dict, restarted_workflow_dict, ["output", "process_resources"]
+        )
+        # Ids will not match, since object is restarted.
+        self.assertNotEqual(restarted_step1_dict["id"], step1_dict["id"])
+        self.assertNotEqual(restarted_step2_dict["id"], step2_dict["id"])
+        self.assertGreater(restarted_step1_dict["created"], step1_dict["created"])
+        self.assertGreater(restarted_step2_dict["created"], step2_dict["created"])
+        restarted_object_sanity_check(
+            step1_dict, restarted_step1_dict, ["id", "permission_group_id", "created"]
+        )
+        restarted_object_sanity_check(
+            step2_dict, restarted_step2_dict, ["id", "permission_group_id", "created"]
+        )
+        self.assertEqual(restarted_workflow_dict["process_resources"], {"cores": 1})
+        self.assertEqual(workflow_dict["process_resources"], {})
+
     @tag_process("test-annotate")
     def test_annotate(self):
         data = self.run_process("test-annotate")
         self.assertIsNotNone(data.entity)
-        dsc = data.entity.descriptor
-        self.assertIn("general", dsc)
-        self.assertIn("species", dsc["general"])
-        self.assertEqual(dsc["general"]["species"], "Valid")
+        self.assertEqual(data.entity.annotations.count(), 1)
+        self.assertAnnotation(data.entity, "general.species", "Valid")
 
     def test_router_handover(self):
         """Test router socket handover.
@@ -165,7 +321,10 @@ class ManagerRunProcessTest(ProcessTestCase):
             "test-annotate-wrong-option", assert_status=Data.STATUS_ERROR
         )
         self.assertEqual(len(data.process_error), 1)
-        self.assertIn("must match one of predefined choices", data.process_error[0])
+        self.assertIn(
+            "The value 'Invalid' is not valid for the field general.species.",
+            data.process_error[0],
+        )
 
     @tag_process("test-annotate-wrong-type")
     def test_annotate_wrong_type(self):
@@ -173,7 +332,9 @@ class ManagerRunProcessTest(ProcessTestCase):
             "test-annotate-wrong-type", assert_status=Data.STATUS_ERROR
         )
         self.assertEqual(len(data.process_error), 1)
-        self.assertIn("is not of type", data.process_error[0])
+        self.assertIn(
+            "The value '15' is not of the expected type 'str'.", data.process_error[0]
+        )
 
     @tag_process("test-annotate-missing-field")
     def test_annotate_missing_field(self):
@@ -181,7 +342,7 @@ class ManagerRunProcessTest(ProcessTestCase):
             "test-annotate-missing-field", assert_status=Data.STATUS_ERROR
         )
         self.assertEqual(len(data.process_error), 1)
-        self.assertIn("definition (invalid) missing in schema", data.process_error[0])
+        self.assertIn("Field 'general.invalid' does not exist", data.process_error[0])
 
     @tag_process("test-annotate-no-entity")
     def test_annotate_no_entity(self):
@@ -271,18 +432,13 @@ class ManagerRunProcessTest(ProcessTestCase):
         )
 
         self.assertEqual(data.status, Data.STATUS_ERROR)
-        print("ERROR", data.process_error[0])
         self.assertIn("must be a valid JSON, current: 1a", data.process_error[0])
 
     @tag_process("test-workflow-1")
     def test_workflow(self):
-        assign_perm("view_collection", self.contributor, self.collection)
-        assign_perm("view_collection", self.group, self.collection)
-
         workflow_data = self.run_process(
             "test-workflow-1", {"param1": "world"}, tags=["test-tag"]
         )
-
         workflow_data.refresh_from_db()
         step1_data = Data.objects.get(process__slug="test-example-1")
         step2_data = Data.objects.get(process__slug="test-example-2")
@@ -317,10 +473,6 @@ class ManagerRunProcessTest(ProcessTestCase):
             {DataDependency.KIND_SUBPROCESS, DataDependency.KIND_IO},
         )
 
-        self.assertTrue(self.contributor.has_perm("flow.view_data", step1_data))
-        # User inherites permission from group
-        self.assertTrue(self.user.has_perm("flow.view_data", step1_data))
-
     @tag_process("test-workflow-2")
     def test_workflow_entity(self):
         with self.preparation_stage():
@@ -329,12 +481,12 @@ class ManagerRunProcessTest(ProcessTestCase):
 
         workflow = self.run_process("test-workflow-2", {"data1": input_data.pk})
 
-        # Check that workflow results are added to the collection.
+        # Check that workflow results are created.
         self.assertEqual(
-            self.collection.data.filter(pk__in=workflow.output["steps"]).count(), 1
+            Data.objects.filter(pk__in=workflow.output["steps"]).count(), 1
         )
-        # self.collection now contains workflow and two "normal" data objects
-        self.assertEqual(self.collection.data.all().count(), 3)
+        # There is a workflow and two "normal" data objects
+        self.assertEqual(Data.objects.all().count(), 3)
 
     @with_docker_executor
     @tag_process("test-docker")
@@ -342,45 +494,101 @@ class ManagerRunProcessTest(ProcessTestCase):
         data = self.run_process("test-docker")
         self.assertEqual(data.output["result"], "OK")
 
+    @patch.object(Manager, "communicate")
+    def test_slug_colision(self, communicate_mock):
+        """Test slug colision resolution.
+
+        Use mocking to force slug colision.
+
+        This checks for a bug in permission architecture that caused exception
+        due to missing permission_group when slug colision occured.
+        """
+
+        async def dummy_coroutine(*args, **kwargs):
+            """Dummy coroutine."""
+            pass
+
+        def pre_save(instance, add):
+            """Set the current slug or raise exception."""
+            current_slug = slugs.pop(0)
+
+            if isinstance(current_slug, Exception):
+                raise current_slug
+
+            instance.slug = current_slug
+            return current_slug
+
+        expected_slug = "my_data_slug"
+        slugs = [IntegrityError("flow_data_slug"), expected_slug]
+        process = Process.objects.create(contributor=self.contributor)
+
+        # INFO: Remove this line and the coroutine above after we stop
+        # supporting Python <= 3.8. Starting with Python 3.8 Mock objects can
+        # be awaited.
+        communicate_mock.side_effect = dummy_coroutine
+
+        with patch.object(ResolweSlugField, "pre_save", side_effect=pre_save):
+            data = Data.objects.create(process=process, contributor=self.contributor)
+        data.refresh_from_db()
+        self.assertEqual(data.slug, expected_slug)
+        self.assertTrue(data.pk is not None)
+        self.assertTrue(data.permission_group is not None)
+
     @with_docker_executor
-    @disable_auto_calls()
     def test_terminate_worker(self):
         process = Process.objects.get(slug="test-terminate")
-        data = Data.objects.create(
-            name="Test data",
-            contributor=self.contributor,
-            process=process,
-        )
 
-        def start_processing(data):
-            async_to_sync(manager.communicate)(data_id=data.pk, run_sync=True)
+        def get_status():
+            return (
+                Data.objects.filter(name="Test terminate")
+                .values_list("status", flat=True)
+                .first()
+            )
 
-        processing_thread = threading.Thread(target=start_processing, args=(data,))
-        processing_thread.start()
+        def wait_status_change(expected_statuses, timeout=20):
+            """Wait for the data object to change status.
 
-        # Wait for up to 5s for process to start.
-        for _ in range(50):
-            sleep(0.1)
-            data.refresh_from_db()
-            with suppress(Data.worker.RelatedObjectDoesNotExist):
-                if data.worker.status == Worker.STATUS_PROCESSING:
-                    break
+            :attr timeout: how many seconds to wait for the status change.
 
-        self.assertEqual(data.worker.status, Worker.STATUS_PROCESSING)
-        data.worker.terminate()
+            :attr expected_statuses: the set of expected statuses to wait for.
 
-        # Give it max 10 seconds to terminate.
-        for _ in range(100):
-            sleep(0.1)
-            data.refresh_from_db()
-            if data.worker.status == Worker.STATUS_COMPLETED:
-                break
+            :returns: the last known status.
+            """
+            start_time = time()
+            status = None
+            while time() - start_time < timeout and status not in expected_statuses:
+                sleep(0.1)
+                status = get_status()
+            return status
 
+        def check_processing():
+            """Start processing data object by saving it."""
+            # Wait up to 20s for process to start.
+            wait_status_change([Data.STATUS_PROCESSING])
+            data = Data.objects.get(name="Test terminate")
+            # Terminate the worker and wait for the process to complete. In the
+            # worst case it will take 60 seconds for the process to complete.
+            logger.debug("test_terminate_worker terminating data '%s'.", data.pk)
+            data.worker.terminate()
+            wait_status_change([Data.STATUS_ERROR, Data.STATUS_DONE])
+            connection.close()
+
+        start_time = time()
+        with ThreadPoolExecutor(max_workers=2) as executor:
+            # Start the check_processing thread.
+            executor.submit(check_processing)
+            # Start processing data object.
+            data = Data.objects.create(
+                name="Test terminate", contributor=self.contributor, process=process
+            )
+
+        # To mark termination as a success it should take less than 50 seconds.
+        # The uninterrupted process would run for 60 seconds.
+        data.refresh_from_db()
         self.assertEqual(data.worker.status, Worker.STATUS_COMPLETED)
         self.assertEqual(data.status, Data.STATUS_ERROR)
         self.assertEqual(data.process_error[0], "Processing was cancelled.")
-        processing_thread.join(timeout=10)
-        self.assertFalse(processing_thread.is_alive())
+        self.assertLess(time() - start_time, 50)
 
     @with_docker_executor
     @tag_process("test-requirements-docker")
@@ -423,9 +631,6 @@ class ManagerRunProcessTest(ProcessTestCase):
         self.run_process("test-cpu-resource-1core")
         self.run_process("test-cpu-resource-2core")
 
-    @unittest.skipIf(
-        settings.FLOW_DOCKER_DISABLE_SECCOMP, "Docker seccomp is disabled."
-    )
     @with_docker_executor
     @tag_process(
         "test-network-resource-enabled",
@@ -475,11 +680,20 @@ class ManagerRunProcessTest(ProcessTestCase):
             "LISTENER_CONNECTION", {}
         )
         port = listener_settings.get("port", 53893)
-        hosts = listener_settings.get("hosts", {"local": "127.0.0.1"})
+        hosts = settings.COMMUNICATION_CONTAINER_LISTENER_CONNECTION
         host = hosts.get("local", next(iter(hosts.values())))
         protocol = settings.FLOW_EXECUTOR.get("LISTENER_CONNECTION", {}).get(
             "protocol", "tcp"
         )
+
+        # Set the status or listener will imediatelly respond with error
+        # status. Make sure to also clear Redis cache for the data object.
+        data.worker.status = Worker.STATUS_PROCESSING
+        data.worker.save()
+        data.status = Data.STATUS_PROCESSING
+        data.save()
+        redis_cache.clear(Data, (data.pk,))
+
         process = subprocess.run(
             [
                 "python",
@@ -496,10 +710,10 @@ class ManagerRunProcessTest(ProcessTestCase):
             stderr=subprocess.PIPE,
             timeout=30,
         )
-        self.assertEqual(process.returncode, 0)
+        self.assertEqual(process.returncode, 0, f"The stderr was: '{process.stderr}'.")
         data.refresh_from_db()
         self.assertEqual(data.output, {})
-        self.assertEqual(data.status, Data.STATUS_DONE)
+        self.assertEqual(data.status, Data.STATUS_PROCESSING)
         self.assertEqual(data.process_error, [])
         self.assertEqual(data.process_info, [])
         # Check that temporary file was not deleted.
@@ -508,18 +722,14 @@ class ManagerRunProcessTest(ProcessTestCase):
     @mock.patch(
         "resolwe.flow.executors.startup_processing_container.asyncio.open_unix_connection"
     )
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
     def test_processing_communication_connection_ok(self, open_mock):
         """Test connection under normal conditions."""
 
         async def main_test():
             """Main test coroutine."""
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager._wait_for_communication_container(None)
+            await asyncio.wait_for(
+                connect_to_communication_container("anypath"), timeout=1
+            )
 
         async def readline():
             """Return PING."""
@@ -541,18 +751,14 @@ class ManagerRunProcessTest(ProcessTestCase):
     @mock.patch(
         "resolwe.flow.executors.startup_processing_container.asyncio.open_unix_connection"
     )
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
-    def test_processing_communication_connection_ok_script(self, open_mock):
+    def test_processing_communication_connection_ok_retry(self, open_mock):
         """Test connection if first attempt fails."""
 
         async def main_test():
             """Main test coroutine."""
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager._wait_for_communication_container(None)
+            await asyncio.wait_for(
+                connect_to_communication_container("anypath"), timeout=3
+            )
 
         async def readline_ok():
             """Return PING."""
@@ -591,8 +797,7 @@ class ManagerRunProcessTest(ProcessTestCase):
         async def main_test():
             """Main test coroutine."""
             current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager._wait_for_communication_container(None)
+            await initialize_connections(current_loop)
 
         async def readline_error():
             """Return ERROR."""
@@ -609,80 +814,9 @@ class ManagerRunProcessTest(ProcessTestCase):
         else:
             open_mock.side_effect = [reader_writer(), reader_writer()]
         loop = asyncio.new_event_loop()
-        with self.assertRaisesRegex(RuntimeError, "Communication .* unreacheable"):
-            loop.run_until_complete(main_test())
-
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.asyncio.open_unix_connection"
-    )
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
-    def test_processing_communication_connection_fail_script(self, open_mock):
-        """Test script behaviour if connection fails."""
-
-        async def main_test():
-            """Main test coroutine."""
-            protocol_mock = mock.Mock(terminate_script=terminate_script)
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            manager.protocol_handler = protocol_mock
-            await manager.run()
-
-        async def terminate_script():
-            """Terminate script mock."""
-
-        async def readline_error():
-            """Return ERROR."""
-            return b"ERROR"
-
-        async def reader_writer():
-            """Return reader-writer tuple mock."""
-            return (mock.Mock(readline=readline_error), None)
-
-        reader_mock = mock.MagicMock()
-        reader_mock.readline.side_effect = [readline_error(), readline_error()]
-        if sys.version_info >= (3, 8):
-            open_mock.return_value = (reader_mock, None)
-        else:
-            open_mock.side_effect = [reader_writer(), reader_writer()]
-        loop = asyncio.new_event_loop()
-        with self.assertRaisesRegex(RuntimeError, "Communication .* unreacheable"):
-            loop.run_until_complete(main_test())
-
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.asyncio.open_unix_connection"
-    )
-    @mock.patch("resolwe.flow.executors.startup_processing_container.socket")
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
-    def test_processing_upload_fail(self, socket_mock, open_mock):
-        """Test startup script if opening upload socket fails."""
-
-        async def main_test():
-            """Main test coroutine."""
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager.run()
-
-        async def readline():
-            """Return PING."""
-            return b"PING"
-
-        async def reader_writer():
-            """Return reader-writer tuple mock."""
-            return (mock.Mock(readline=readline), None)
-
-        socket_mock.socket.return_value.connect.side_effect = RuntimeError
-        if sys.version_info >= (3, 8):
-            open_mock.return_value = (mock.Mock(readline=readline), None)
-        else:
-            open_mock.return_value = reader_writer()
-        loop = asyncio.new_event_loop()
-        with self.assertRaises(RuntimeError):
+        with self.assertRaisesRegex(
+            ConnectionError, "Timeout connecting to communication container."
+        ):
             loop.run_until_complete(main_test())
 
     @mock.patch(
@@ -692,103 +826,14 @@ class ManagerRunProcessTest(ProcessTestCase):
     @mock.patch(
         "resolwe.flow.executors.startup_processing_container.asyncio.start_unix_server"
     )
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
-    def test_processing_script_connection_fail(
-        self, server_mock, socket_mock, open_mock
-    ):
-        """Test startup script if opening processing script socket fails."""
-
-        async def main_test():
-            """Main test coroutine."""
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager.run()
-
-        async def readline():
-            """Return PING."""
-            return b"PING"
-
-        async def reader_writer():
-            """Return reader-writer tuple mock."""
-            return (mock.Mock(readline=readline), None)
-
-        socket_mock.socket.return_value.connect.side_effect = None
-        if sys.version_info >= (3, 8):
-            open_mock.return_value = (mock.Mock(readline=readline), None)
-        else:
-            open_mock.return_value = reader_writer()
-        loop = asyncio.new_event_loop()
-        server_mock.side_effect = ValueError("test")
-        with self.assertRaisesRegex(ValueError, "test"):
-            loop.run_until_complete(main_test())
-
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.asyncio.open_unix_connection"
-    )
-    @mock.patch("resolwe.flow.executors.startup_processing_container.socket")
-    @mock.patch("resolwe.flow.executors.startup_processing_container.create_task")
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.asyncio.start_unix_server"
-    )
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
-    def test_processing_create_task_called(
-        self, server_mock, create_mock, socket_mock, open_mock
-    ):
-        """Test that upload timer task is created."""
-
-        async def main_test():
-            """Main test coroutine."""
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager.run()
-
-        async def readline():
-            """Return PING."""
-            return b"PING"
-
-        async def reader_writer():
-            """Return reader-writer tuple mock."""
-            return (mock.Mock(readline=readline), None)
-
-        socket_mock.socket.return_value.connect.side_effect = None
-        if sys.version_info >= (3, 8):
-            open_mock.return_value = (mock.Mock(readline=readline), None)
-        else:
-            open_mock.return_value = reader_writer()
-        loop = asyncio.new_event_loop()
-        create_mock.side_effect = ValueError("test")
-        with self.assertRaisesRegex(ValueError, "test"):
-            loop.run_until_complete(main_test())
-
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.asyncio.open_unix_connection"
-    )
-    @mock.patch("resolwe.flow.executors.startup_processing_container.socket")
-    @mock.patch("resolwe.flow.executors.startup_processing_container.create_task")
-    @mock.patch("resolwe.flow.executors.startup_processing_container.ProtocolHandler")
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.asyncio.start_unix_server"
-    )
-    @mock.patch(
-        "resolwe.flow.executors.startup_processing_container.constants.CONTAINER_TIMEOUT",
-        2,
-    )
-    def test_processing_communicate(
-        self, server_mock, protocol_mock, create_mock, socket_mock, open_mock
-    ):
+    def test_processing_communicate(self, server_mock, socket_mock, open_mock):
         """Test that communicate is called."""
 
         async def main_test():
             """Main test coroutine."""
-            current_loop = asyncio.get_event_loop()
-            manager = ProcessingManager(current_loop)
-            await manager.run()
+            await asyncio.wait_for(
+                connect_to_communication_container("anypath"), timeout=1
+            )
 
         async def readline():
             """Return PING."""
@@ -803,7 +848,6 @@ class ManagerRunProcessTest(ProcessTestCase):
             open_mock.return_value = (mock.Mock(readline=readline), None)
         else:
             open_mock.return_value = reader_writer()
-        protocol_mock.return_value.communicate.side_effect = ValueError("test")
+        # protocol_mock.return_value.communicate.side_effect = ValueError("test")
         loop = asyncio.new_event_loop()
-        with self.assertRaisesRegex(ValueError, "test"):
-            loop.run_until_complete(main_test())
+        loop.run_until_complete(main_test())
