@@ -2,20 +2,30 @@
 import datetime
 
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import Group
 from django.utils.timezone import get_current_timezone
 
-from guardian.shortcuts import assign_perm
 from rest_framework import status
 from rest_framework.test import APIRequestFactory, force_authenticate
 
-from resolwe.flow.models import Collection, Data, DescriptorSchema, Entity, Process
+from resolwe.flow.models import (
+    Collection,
+    Data,
+    DescriptorSchema,
+    Entity,
+    Process,
+    Relation,
+    RelationType,
+)
 from resolwe.flow.views import (
     CollectionViewSet,
     DataViewSet,
     DescriptorSchemaViewSet,
     EntityViewSet,
     ProcessViewSet,
+    RelationViewSet,
 )
+from resolwe.permissions.models import Permission, get_anonymous_user
 from resolwe.test import TestCase
 
 factory = APIRequestFactory()
@@ -44,6 +54,7 @@ class BaseViewSetFiltersTest(TestCase):
             first_name="John",
             last_name="Williams",
         )
+        cls.group = Group.objects.create(name="Test group")
 
     def setUp(self):
         self.viewset = self.viewset_class.as_view(
@@ -131,8 +142,26 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
                 description="User's description",
                 tags=["second-tag"],
             ),
+            Collection.objects.create(
+                name="Public test collection",
+                slug="public-collection",
+                contributor=cls.admin,
+            ),
         ]
 
+        cls.entity1 = Entity.objects.create(
+            contributor=cls.contributor, collection=cls.collections[1]
+        )
+        cls.entity2 = Entity.objects.create(
+            contributor=cls.contributor, collection=cls.collections[2]
+        )
+        cls.entity3 = Entity.objects.create(
+            contributor=cls.contributor, collection=cls.collections[2]
+        )
+        cls.entity1.save()
+        cls.entity2.save()
+        cls.entity3.save()
+
         tzone = get_current_timezone()
         cls.collections[0].created = datetime.datetime(2016, 7, 30, 0, 59, tzinfo=tzone)
         cls.collections[0].save()
@@ -141,13 +170,13 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         cls.collections[2].created = datetime.datetime(2016, 7, 30, 2, 59, tzinfo=tzone)
         cls.collections[2].save()
 
-        assign_perm("owner_collection", cls.admin, cls.collections[0])
-        assign_perm("view_collection", cls.user, cls.collections[0])
-        assign_perm("owner_collection", cls.admin, cls.collections[1])
-        assign_perm("view_collection", cls.user, cls.collections[1])
-        assign_perm("view_collection", cls.user, cls.collections[2])
-        assign_perm("edit_collection", cls.user, cls.collections[2])
-        assign_perm("owner_collection", cls.user, cls.collections[2])
+        cls.collections[0].set_permission(Permission.OWNER, cls.admin)
+        cls.collections[0].set_permission(Permission.VIEW, cls.user)
+        cls.collections[1].set_permission(Permission.OWNER, cls.admin)
+        cls.collections[1].set_permission(Permission.VIEW, cls.user)
+        cls.collections[2].set_permission(Permission.OWNER, cls.user)
+        cls.collections[2].set_permission(Permission.OWNER, cls.group)
+        cls.collections[3].set_permission(Permission.VIEW, get_anonymous_user())
 
         cls.viewset_class = CollectionViewSet
 
@@ -163,7 +192,7 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
     def test_filter_slug(self):
         self._check_filter({"slug": "test-collection-0"}, [self.collections[0]])
         self._check_filter(
-            {"slug__in": "test-collection-1,user-collection"}, self.collections[1:]
+            {"slug__in": "test-collection-1,user-collection"}, self.collections[1:3]
         )
 
     def test_filter_name(self):
@@ -179,7 +208,7 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
             {"name__istartswith": "test collection"}, self.collections[:2]
         )
 
-        self._check_filter({"name__contains": "test"}, [self.collections[2]])
+        self._check_filter({"name__contains": "test"}, self.collections[2:])
         self._check_filter({"name__icontains": "test"}, self.collections)
 
     def test_filter_description(self):
@@ -205,7 +234,7 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         )
         self._check_filter(
             {"contributor__in": "{},{}".format(self.contributor.pk, self.user.pk)},
-            self.collections,
+            self.collections[:3],
         )
 
     def test_filter_owners(self):
@@ -236,6 +265,14 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter(
             {"permission": "edit"}, [self.collections[2]], user=self.user
         )
+        self._check_filter(
+            {"permission": "view"},
+            self.collections,
+            user=self.user,
+        )
+        self._check_filter(
+            {"permission": "owner"}, [self.collections[2]], user=self.user
+        )
 
     def test_filter_created(self):
         self._check_filter({"created": "2016-07-30T00:59:00"}, self.collections[:1])
@@ -254,19 +291,19 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         )
         self._check_filter(
             {"modified__gt": self.collections[1].modified.isoformat()},
-            self.collections[2:],
+            self.collections[2:3],
         )
         self._check_filter(
             {"modified__gte": self.collections[1].modified.isoformat()},
-            self.collections[1:],
+            self.collections[1:3],
         )
         self._check_filter(
             {"modified__lt": self.collections[1].modified.isoformat()},
-            self.collections[:1],
+            self.collections[:1] + [self.collections[3]],
         )
         self._check_filter(
             {"modified__lte": self.collections[1].modified.isoformat()},
-            self.collections[:2],
+            self.collections[:2] + [self.collections[3]],
         )
 
     def test_filter_text(self):
@@ -281,8 +318,12 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"text": "Miller"}, self.collections[:2])
 
         # By owner.
-        self._check_filter({"text": "james"}, self.collections[:2])
-        self._check_filter({"text": "Smith"}, self.collections[:2])
+        self._check_filter(
+            {"text": "james"}, self.collections[:2] + [self.collections[3]]
+        )
+        self._check_filter(
+            {"text": "Smith"}, self.collections[:2] + [self.collections[3]]
+        )
 
         # By description.
         self._check_filter(
@@ -334,6 +375,34 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         )
         self.assertEqual(result.data[0]["id"], self.collections[0].pk)
 
+    def test_filter_entity_count(self):
+        self._check_filter(
+            {"entity_count": 0}, [self.collections[0], self.collections[3]]
+        )
+        self._check_filter({"entity_count": 2}, [self.collections[2]])
+        self._check_filter({"entity_count__gt": 1}, [self.collections[2]])
+        self._check_filter(
+            {"entity_count__lt": 1}, [self.collections[0], self.collections[3]]
+        )
+        self._check_filter(
+            {"entity_count__gte": 1}, [self.collections[1], self.collections[2]]
+        )
+        self._check_filter(
+            {"entity_count__lte": 1},
+            [self.collections[0], self.collections[1], self.collections[3]],
+        )
+
+    def test_filter_group(self):
+        self._check_filter({"group": self.group.pk}, [self.collections[2]])
+
+    def test_filter_shared_with_me(self):
+        self._check_filter(
+            {"shared_with_me": True}, self.collections[:3], user=self.user
+        )
+        self._check_filter(
+            {"shared_with_me": False}, [self.collections[3]], user=self.user
+        )
+
 
 class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
     @classmethod
@@ -407,8 +476,25 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
                 description="User's description",
                 tags=["second-tag"],
             ),
+            Entity.objects.create(
+                name="Public test entity",
+                slug="public-entity",
+                contributor=cls.user,
+            ),
         ]
 
+        cls.rel_type = RelationType.objects.create(name="series", ordered=True)
+        cls.relation1 = Relation.objects.create(
+            type=cls.rel_type, collection=cls.collection1, contributor=cls.contributor
+        )
+        cls.relation1.entities.set(cls.entities[:1])
+        cls.relation1.save()
+        cls.relation2 = Relation.objects.create(
+            type=cls.rel_type, collection=cls.collection2, contributor=cls.contributor
+        )
+        cls.relation2.entities.set(cls.entities[1:])
+        cls.relation2.save()
+
         tzone = get_current_timezone()
         cls.entities[0].created = datetime.datetime(2016, 7, 30, 0, 59, tzinfo=tzone)
         cls.entities[0].save()
@@ -417,13 +503,13 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         cls.entities[2].created = datetime.datetime(2016, 7, 30, 2, 59, tzinfo=tzone)
         cls.entities[2].save()
 
-        assign_perm("owner_entity", cls.admin, cls.entities[0])
-        assign_perm("view_entity", cls.user, cls.entities[0])
-        assign_perm("owner_entity", cls.admin, cls.entities[1])
-        assign_perm("view_entity", cls.user, cls.entities[1])
-        assign_perm("owner_entity", cls.user, cls.entities[2])
-        assign_perm("view_entity", cls.user, cls.entities[2])
-        assign_perm("edit_entity", cls.user, cls.entities[2])
+        cls.collection1.set_permission(Permission.OWNER, cls.admin)
+        cls.collection1.set_permission(Permission.VIEW, cls.user)
+        cls.collection2.set_permission(Permission.OWNER, cls.admin)
+        cls.collection2.set_permission(Permission.VIEW, cls.user)
+        cls.entities[2].set_permission(Permission.OWNER, cls.user)
+        cls.entities[2].set_permission(Permission.VIEW, cls.group)
+        cls.entities[3].set_permission(Permission.VIEW, get_anonymous_user())
 
         cls.viewset_class = EntityViewSet
 
@@ -438,7 +524,9 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
 
     def test_filter_slug(self):
         self._check_filter({"slug": "test-entity-0"}, [self.entities[0]])
-        self._check_filter({"slug__in": "test-entity-1,user-entity"}, self.entities[1:])
+        self._check_filter(
+            {"slug__in": "test-entity-1,user-entity"}, self.entities[1:3]
+        )
 
     def test_filter_name(self):
         self._check_filter({"name": "Test entity 1"}, [self.entities[1]])
@@ -449,7 +537,7 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"name__startswith": "Test entity"}, self.entities[:2])
         self._check_filter({"name__istartswith": "test entity"}, self.entities[:2])
 
-        self._check_filter({"name__contains": "test"}, [self.entities[2]])
+        self._check_filter({"name__contains": "test"}, self.entities[2:])
         self._check_filter({"name__icontains": "test"}, self.entities)
 
     def test_filter_description(self):
@@ -486,7 +574,7 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         # Filter by username.
         self._check_filter({"contributor_name": "contributor"}, self.entities[:2])
         # Filter by combination of first and last name.
-        self._check_filter({"contributor_name": "John Williams"}, [self.entities[2]])
+        self._check_filter({"contributor_name": "John Williams"}, self.entities[2:])
 
     def test_filter_owners_name(self):
         # Filter by first name.
@@ -500,6 +588,12 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
 
     def test_filter_permission(self):
         self._check_filter({"permission": "edit"}, [self.entities[2]], user=self.user)
+        self._check_filter(
+            {"permission": "view"},
+            self.entities,
+            user=self.user,
+        )
+        self._check_filter({"permission": "owner"}, [self.entities[2]], user=self.user)
 
     def test_filter_created(self):
         self._check_filter({"created": "2016-07-30T00:59:00"}, self.entities[:1])
@@ -513,16 +607,18 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
             {"modified": self.entities[0].modified.isoformat()}, self.entities[:1]
         )
         self._check_filter(
-            {"modified__gt": self.entities[1].modified.isoformat()}, self.entities[2:]
+            {"modified__gt": self.entities[1].modified.isoformat()}, [self.entities[2]]
         )
         self._check_filter(
-            {"modified__gte": self.entities[1].modified.isoformat()}, self.entities[1:]
+            {"modified__gte": self.entities[1].modified.isoformat()}, self.entities[1:3]
         )
         self._check_filter(
-            {"modified__lt": self.entities[1].modified.isoformat()}, self.entities[:1]
+            {"modified__lt": self.entities[1].modified.isoformat()},
+            self.entities[:1] + [self.entities[3]],
         )
         self._check_filter(
-            {"modified__lte": self.entities[1].modified.isoformat()}, self.entities[:2]
+            {"modified__lte": self.entities[1].modified.isoformat()},
+            self.entities[:2] + [self.entities[3]],
         )
 
     def test_filter_collection(self):
@@ -563,7 +659,7 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"text": "Test entity 1"}, [self.entities[1]])
         self._check_filter({"text": "Test"}, self.entities)
         self._check_filter({"text": "test"}, self.entities)
-        self._check_filter({"text": "user"}, [self.entities[2]])
+        self._check_filter({"text": "user"}, self.entities[2:])
 
         # By contributor.
         self._check_filter({"text": "joe"}, self.entities[:2])
@@ -579,7 +675,7 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         )
         self._check_filter({"text": "my"}, self.entities[:2])
         self._check_filter({"text": "my description"}, [self.entities[0]])
-        self._check_filter({"text": "user"}, [self.entities[2]])
+        self._check_filter({"text": "user"}, self.entities[2:])
 
         # By descriptor.
         self._check_filter({"text": "genialis"}, [self.entities[0]])
@@ -621,12 +717,45 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         )
         self.assertEqual(result.data[0]["id"], self.entities[0].pk)
 
+    def test_order_by_collection_name(self):
+        """Check entities can be ordered by the collection name."""
+        request = factory.get("/", {"ordering": "collection__name"}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        received = [
+            (item["collection"] or {}).get("name", None) for item in response.data
+        ]
+        expected = ["My collection", "Other collection", None, None]
+        self.assertEqual(received, expected)
+
+        request = factory.get("/", {"ordering": "-collection__name"}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        received = [
+            (item["collection"] or {}).get("name", None) for item in response.data
+        ]
+        expected.reverse()
+        self.assertEqual(received, expected)
+
+    def test_filter_by_relation(self):
+        self._check_filter({"relation_id": self.relation1.id}, self.entities[:1])
+        self._check_filter({"relation_id": self.relation2.id}, self.entities[1:])
+        self._check_filter({"relation_id": 42}, [])
+
+    def test_filter_group(self):
+        self._check_filter({"group": self.group.pk}, [self.entities[2]])
+
+    def test_filter_shared_with_me(self):
+        self._check_filter({"shared_with_me": True}, self.entities[:3], user=self.user)
+        self._check_filter(
+            {"shared_with_me": False}, [self.entities[3]], user=self.user
+        )
+
 
 class DataViewSetFiltersTest(BaseViewSetFiltersTest):
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
-
         tzone = get_current_timezone()
 
         descriptor_schema = DescriptorSchema.objects.create(
@@ -742,8 +871,29 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
                 finished=datetime.datetime(2016, 7, 31, 2, 30, tzinfo=tzone),
                 tags=["second-tag"],
             ),
+            Data.objects.create(
+                name="Public data",
+                slug="public-data",
+                contributor=cls.user,
+                process=cls.proc2,
+                status=Data.STATUS_DONE,
+                started=datetime.datetime(2016, 7, 31, 2, 0, tzinfo=tzone),
+                finished=datetime.datetime(2016, 7, 31, 2, 30, tzinfo=tzone),
+            ),
         ]
 
+        cls.rel_type = RelationType.objects.create(name="series", ordered=True)
+        cls.relation1 = Relation.objects.create(
+            type=cls.rel_type, collection=cls.collection1, contributor=cls.contributor
+        )
+        cls.relation1.entities.set([cls.entity1, cls.entity2])
+        cls.relation1.save()
+        cls.relation2 = Relation.objects.create(
+            type=cls.rel_type, collection=cls.collection2, contributor=cls.contributor
+        )
+        cls.relation2.entities.set([cls.entity2])
+        cls.relation2.save()
+
         cls.data[0].created = datetime.datetime(2016, 7, 30, 0, 59, tzinfo=tzone)
         cls.data[0].save()
         cls.data[1].created = datetime.datetime(2016, 7, 30, 1, 59, tzinfo=tzone)
@@ -751,13 +901,13 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
         cls.data[2].created = datetime.datetime(2016, 7, 30, 2, 59, tzinfo=tzone)
         cls.data[2].save()
 
-        assign_perm("owner_data", cls.admin, cls.data[0])
-        assign_perm("view_data", cls.user, cls.data[0])
-        assign_perm("owner_data", cls.admin, cls.data[1])
-        assign_perm("view_data", cls.user, cls.data[1])
-        assign_perm("owner_data", cls.user, cls.data[2])
-        assign_perm("view_data", cls.user, cls.data[2])
-        assign_perm("edit_data", cls.user, cls.data[2])
+        cls.collection1.set_permission(Permission.OWNER, cls.admin)
+        cls.collection1.set_permission(Permission.VIEW, cls.user)
+        cls.collection2.set_permission(Permission.OWNER, cls.admin)
+        cls.collection2.set_permission(Permission.VIEW, cls.user)
+        cls.data[2].set_permission(Permission.OWNER, cls.user)
+        cls.data[2].set_permission(Permission.VIEW, cls.group)
+        cls.data[3].set_permission(Permission.VIEW, get_anonymous_user())
 
         cls.viewset_class = DataViewSet
 
@@ -810,7 +960,7 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
         # Filter by username.
         self._check_filter({"contributor_name": "contributor"}, self.data[:2])
         # Filter by combination of first and last name.
-        self._check_filter({"contributor_name": "John Williams"}, [self.data[2]])
+        self._check_filter({"contributor_name": "John Williams"}, self.data[2:])
 
     def test_filter_owners_name(self):
         # Filter by first name.
@@ -820,10 +970,16 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
         # Filter by username.
         self._check_filter({"owners_name": "admin"}, self.data[:2])
         # Filter by combination of first and last name.
-        self._check_filter({"owners_name": "John Williams"}, [self.data[2]])
+        self._check_filter({"owners_name": "John Williams"}, self.data[2:])
 
     def test_filter_permission(self):
-        self._check_filter({"permission": "edit"}, [self.data[2]], user=self.user)
+        self._check_filter({"permission": "edit"}, self.data[2:], user=self.user)
+        self._check_filter(
+            {"permission": "view"},
+            self.data,
+            user=self.user,
+        )
+        self._check_filter({"permission": "owner"}, self.data[2:], user=self.user)
 
     def test_filter_created(self):
         self._check_filter({"created": "2016-07-30T00:59:00"}, [self.data[0]])
@@ -837,16 +993,18 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
             {"modified": self.data[0].modified.isoformat()}, [self.data[0]]
         )
         self._check_filter(
-            {"modified__gt": self.data[1].modified.isoformat()}, self.data[2:]
+            {"modified__gt": self.data[1].modified.isoformat()}, [self.data[2]]
         )
         self._check_filter(
-            {"modified__gte": self.data[1].modified.isoformat()}, self.data[1:]
+            {"modified__gte": self.data[1].modified.isoformat()}, self.data[1:3]
         )
         self._check_filter(
-            {"modified__lt": self.data[1].modified.isoformat()}, self.data[:1]
+            {"modified__lt": self.data[1].modified.isoformat()},
+            self.data[:1] + [self.data[3]],
         )
         self._check_filter(
-            {"modified__lte": self.data[1].modified.isoformat()}, self.data[:2]
+            {"modified__lte": self.data[1].modified.isoformat()},
+            self.data[:2] + [self.data[3]],
         )
 
     def test_filter_started(self):
@@ -905,7 +1063,7 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
             [self.data[0]],
         )
         self._check_filter(
-            {"entity__in": "{},{}".format(self.entity1.pk, self.entity2.pk, "999999")},
+            {"entity__in": "{},{}".format(self.entity1.pk, self.entity2.pk)},
             self.data[:2],
         )
         self._check_filter(
@@ -1008,6 +1166,24 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
         result = self._check_filter({"text": "data", "ordering": "id"}, self.data)
         self.assertEqual(result.data[0]["id"], self.data[0].pk)
 
+    def test_ordering_entity_name(self):
+        """Check that data can be ordered by the entity name."""
+        expected = ["My entity", "Other entity", None, None]
+
+        request = factory.get("/", {"ordering": "entity__name"}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        received = [(item.get("entity") or {}).get("name") for item in response.data]
+        self.assertTrue(status.is_success(response.status_code))
+        self.assertEqual(expected, received)
+
+        request = factory.get("/", {"ordering": "-entity__name"}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        received = [(item.get("entity") or {}).get("name") for item in response.data]
+        self.assertTrue(status.is_success(response.status_code))
+        self.assertEqual(list(reversed(expected)), received)
+
     def test_nonexisting_parameter(self):
         response = self._check_filter({"foo": "bar"}, [], expected_status_code=400)
 
@@ -1016,6 +1192,20 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
             r"Unsupported parameter\(s\): foo. Please use a combination of: .*",
         )
 
+    def test_filter_by_relation(self):
+        self._check_filter(
+            {"relation_id": self.relation1.id}, [self.data[0], self.data[1]]
+        )
+        self._check_filter({"relation_id": self.relation2.id}, [self.data[1]])
+        self._check_filter({"relation_id": 42}, [])
+
+    def test_filter_group(self):
+        self._check_filter({"group": self.group.pk}, [self.data[2]])
+
+    def test_filter_shared_with_me(self):
+        self._check_filter({"shared_with_me": True}, self.data[:3], user=self.user)
+        self._check_filter({"shared_with_me": False}, [self.data[3]], user=self.user)
+
 
 class DescriptorSchemaViewSetFiltersTest(BaseViewSetFiltersTest):
     def setUp(self):
@@ -1149,3 +1339,33 @@ class ProcessViewSetFiltersTest(BaseViewSetFiltersTest):
 
         self._check_filter({"is_active": "true"}, [self.proc_2])
         self._check_filter({"is_active": "false"}, [self.proc_1])
+
+
+class RelationViewSetFiltersTest(BaseViewSetFiltersTest):
+    @classmethod
+    def setUpTestData(cls):
+        super().setUpTestData()
+
+        cls.collection_1 = Collection.objects.create(
+            contributor=cls.contributor,
+            tags=["first-tag"],
+        )
+
+        cls.collection_2 = Collection.objects.create(
+            contributor=cls.contributor,
+            tags=["second-tag"],
+        )
+
+        cls.rel_type = RelationType.objects.create(name="series", ordered=True)
+        cls.relation_1 = Relation.objects.create(
+            type=cls.rel_type, collection=cls.collection_1, contributor=cls.contributor
+        )
+
+        cls.relation_2 = Relation.objects.create(
+            type=cls.rel_type, collection=cls.collection_2, contributor=cls.contributor
+        )
+
+        cls.viewset_class = RelationViewSet
+
+    def test_filter_tags(self):
+        self._check_filter({"tags": "first-tag"}, [self.relation_1])
