@@ -1,13 +1,8 @@
 # pylint: disable=missing-docstring
 import os
-from pathlib import Path
 
 from asgiref.sync import async_to_sync
 
-from django.test import override_settings
-
-from guardian.shortcuts import assign_perm
-
 from resolwe.flow.managers import manager
 from resolwe.flow.managers.utils import disable_auto_calls
 from resolwe.flow.models import (
@@ -17,6 +12,7 @@ from resolwe.flow.models import (
     DescriptorSchema,
     Process,
 )
+from resolwe.permissions.models import Permission
 from resolwe.test import ProcessTestCase, TransactionTestCase
 
 PROCESSES_DIR = os.path.join(os.path.dirname(__file__), "processes")
@@ -27,8 +23,9 @@ class TestManager(ProcessTestCase):
         super().setUp()
 
         self.collection = Collection.objects.create(contributor=self.contributor)
-
         self._register_schemas(processes_paths=[PROCESSES_DIR])
+        manager._processes_ignore = None
+        manager._processes_allow = None
 
     def test_create_data(self):
         """Test that manager is run when new object is created."""
@@ -38,10 +35,44 @@ class TestManager(ProcessTestCase):
             contributor=self.contributor,
             process=process,
         )
-
         data.refresh_from_db()
         self.assertEqual(data.status, Data.STATUS_DONE)
 
+    def test_ignore_allow_list(self):
+        process = Process.objects.filter(slug="test-min").latest()
+
+        # Ignored processes should not trigger processing.
+        manager._processes_ignore = ["test-min"]
+        data = Data.objects.create(
+            name="Test data",
+            contributor=self.contributor,
+            process=process,
+        )
+        data.refresh_from_db()
+        self.assertEqual(data.status, Data.STATUS_RESOLVING)
+
+        # Ignore should have precedence.
+        manager._processes_ignore = ["test-min"]
+        manager._processes_allow = ["test-min"]
+        data = Data.objects.create(
+            name="Test data",
+            contributor=self.contributor,
+            process=process,
+        )
+        data.refresh_from_db()
+        self.assertEqual(data.status, Data.STATUS_RESOLVING)
+
+        # Allowing some processes shoud disable others.
+        manager._processes_ignore = None
+        manager._processes_allow = ["test-something-else"]
+        data = Data.objects.create(
+            name="Test data",
+            contributor=self.contributor,
+            process=process,
+        )
+        data.refresh_from_db()
+        self.assertEqual(data.status, Data.STATUS_RESOLVING)
+
     def test_spawned_process(self):
         """Test that manager is run for spawned processes and permissions are copied."""
         DescriptorSchema.objects.create(
@@ -53,7 +84,9 @@ class TestManager(ProcessTestCase):
         spawned_process.entity_descriptor_schema = "test-schema"
         spawned_process.save()
 
-        assign_perm("view_collection", self.user, self.collection)
+        # Make sure user can spawn the process.
+        spawned_process.set_permission(Permission.VIEW, self.contributor)
+        self.collection.set_permission(Permission.VIEW, self.user)
         Data.objects.create(
             name="Test data",
             contributor=self.contributor,
@@ -66,7 +99,7 @@ class TestManager(ProcessTestCase):
 
         # Check that permissions are inherited.
         child = Data.objects.last()
-        self.assertTrue(self.user.has_perm("view_data", child))
+        self.assertTrue(self.user.has_perm(Permission.VIEW, child))
         self.assertEqual(child.collection.pk, self.collection.pk)
         self.assertEqual(child.entity.collection.pk, self.collection.pk)
 
@@ -99,17 +132,6 @@ class TestManager(ProcessTestCase):
             {DataDependency.KIND_SUBPROCESS},
         )
 
-    def test_communicate(self):
-        """Make sure communicate can serialize settings.
-
-        When object of PosixPath is used in settings the call to communicate
-        fails due to object not being JSON serializable by default.
-        """
-        with override_settings(FLOW_EXECUTOR={"DATA_DIR": Path("/some/path")}):
-            async_to_sync(manager.communicate)(run_sync=True)
-        # Make sure original settings are saved.
-        async_to_sync(manager.communicate)(run_sync=True)
-
     def test_dependencies(self):
         """Test that manager handles dependencies correctly."""
         process_parent = Process.objects.filter(slug="test-dependency-parent").latest()
@@ -210,7 +232,8 @@ class TransactionTestManager(TransactionTestCase):
 
         self.assertEqual(Data.objects.filter(status=Data.STATUS_RESOLVING).count(), 4)
 
-        # Process only one object.
+        # Allow unfinished data objects to exist when checking for execution
+        # barrier condition in the dispatcher.
         async_to_sync(manager.communicate)(data_id=data_1.pk, run_sync=True)
 
         data_1.refresh_from_db()
