@@ -23,6 +23,7 @@ from resolwe.flow.models import (
 from resolwe.flow.models.data import Data, DataDependency, hydrate_size, render_template
 from resolwe.flow.models.utils import hydrate_input_references, referenced_files
 from resolwe.flow.views import DataViewSet
+from resolwe.observers.models import BackgroundTask
 from resolwe.permissions.models import Permission
 from resolwe.test import TestCase, TransactionTestCase
 from resolwe.test.utils import create_data_location, save_storage
@@ -137,6 +138,20 @@ class DataModelTest(TestCase):
         Data.objects.all().delete_chunked(chunk_size=10)
         self.assertFalse(Data.objects.exists())
 
+    def test_move_to_entity(self):
+        # Create data outside container and move it to the entity in the collection.
+        process = Process.objects.create(contributor=self.contributor)
+        data = Data.objects.create(contributor=self.contributor, process=process)
+        self.assertIsNone(data.entity)
+        self.assertIsNone(data.collection)
+        collection = Collection.objects.create(contributor=self.contributor)
+        entity = Entity.objects.create(
+            contributor=self.contributor, collection=collection
+        )
+        data.move_to_entity(entity)
+        self.assertEqual(data.entity, entity)
+        self.assertEqual(data.collection, collection)
+
     def test_trim_name(self):
         process = Process.objects.create(
             contributor=self.contributor, data_name="test" * 50
@@ -605,7 +620,7 @@ class GetOrCreateTestCase(APITestCase):
         self.assertNotEqual(response.data["id"], self.data.pk)
 
 
-class DuplicateTestCase(TestCase):
+class DuplicateTestCase(TransactionTestCase):
     def test_data_duplicate(self):
         process1 = Process.objects.create(
             contributor=self.user,
@@ -647,7 +662,8 @@ class DuplicateTestCase(TestCase):
         data2.migration_history.create(migration="migration_2")
 
         # Duplicate.
-        duplicates = Data.objects.filter(id=data2.id).duplicate(self.contributor)
+        task = Data.objects.filter(id=data2.id).duplicate(self.contributor)
+        duplicates = Data.objects.filter(pk__in=task.result())
         self.assertEqual(len(duplicates), 1)
         duplicate = duplicates[0]
         self.assertTrue(duplicate.is_duplicate())
@@ -761,8 +777,10 @@ class DuplicateTestCase(TestCase):
         data.migration_history.create(migration="migration_1")
 
         # Duplicate.
-        duplicate = data.duplicate(self.contributor)
-        duplicate_of_duplicate = duplicate.duplicate(self.contributor)
+        task = data.duplicate(self.contributor)
+        duplicate = Data.objects.get(pk__in=task.result())
+        task = duplicate.duplicate(self.contributor)
+        duplicate_of_duplicate = Data.objects.get(pk__in=task.result())
 
         self.assertEqual(
             duplicate.contributor.id, duplicate_of_duplicate.contributor.id
@@ -808,6 +826,11 @@ class DuplicateTestCase(TestCase):
         )
 
     def test_input_rewiring(self):
+        """Auto calls must be disabled or duplication with multiple objects will fail.
+
+        The first object will call the dispatcher and wait for the workers to finish,
+        while the second one will wait for the first.
+        """
         process1 = Process.objects.create(
             contributor=self.user,
             type="data:test:first:",
@@ -821,14 +844,19 @@ class DuplicateTestCase(TestCase):
                 {"name": "data_list_field", "type": "list:data:test:first:"},
             ],
         )
+        entity = Entity.objects.create(contributor=self.user, name="entity")
+        entity.set_permission(Permission.EDIT, self.contributor)
+        entity.set_permission(Permission.EDIT, self.user)
 
         data1 = Data.objects.create(
             contributor=self.user,
             process=process1,
             status=Data.STATUS_DONE,
+            entity=entity,
         )
 
-        should_not_be_rewritten = data1.duplicate(self.user)
+        task = data1.duplicate(self.user)
+        should_not_be_rewritten = Data.objects.get(pk__in=task.result())
 
         data2 = Data.objects.create(
             contributor=self.user,
@@ -839,15 +867,13 @@ class DuplicateTestCase(TestCase):
                 "data_list_field": [data1.id, should_not_be_rewritten.id],
             },
             status=Data.STATUS_DONE,
+            entity=entity,
         )
 
-        # Duplicate.
-        duplicate1, duplicate2 = (
-            Data.objects.filter(id__in=[data1.id, data2.id])
-            .order_by("id")
-            .duplicate(self.contributor)
-        ).order_by("id")
-
+        task = Data.objects.filter(id__in=[data1.id, data2.id]).duplicate(
+            self.contributor
+        )
+        duplicate1, duplicate2 = Data.objects.filter(pk__in=task.result(timeout=10))
         self.assertEqual(duplicate2.input["data_field1"], duplicate1.id)
         self.assertEqual(duplicate2.input["data_field2"], should_not_be_rewritten.id)
         self.assertEqual(
@@ -864,11 +890,14 @@ class DuplicateTestCase(TestCase):
             contributor=self.user, process=process, status=Data.STATUS_DONE
         )
 
-        with self.assertRaisesMessage(
-            ValidationError,
-            "Data object must have done or error status to be duplicated",
-        ):
-            Data.objects.filter(id__in=[data.id, data2.id]).duplicate(self.contributor)
+        task = Data.objects.filter(id__in=[data.id, data2.id]).duplicate(
+            self.contributor
+        )
+        task.wait(final_statuses=[BackgroundTask.STATUS_ERROR])
+        self.assertCountEqual(
+            task.output,
+            ["Data object must have done or error status to be duplicated."],
+        )
 
     def test_data_long_name(self):
         process = Process.objects.create(contributor=self.user)
@@ -881,8 +910,8 @@ class DuplicateTestCase(TestCase):
             status=Data.STATUS_DONE,
         )
 
-        duplicate = Data.objects.filter(id=data.id).duplicate(self.contributor)[0]
-
+        task = Data.objects.filter(id=data.id).duplicate(self.contributor)
+        duplicate = Data.objects.get(pk__in=task.result())
         self.assertTrue(duplicate.name.startswith("Copy of "))
         self.assertTrue(duplicate.name.endswith("..."))
         self.assertEqual(len(duplicate.name), Data._meta.get_field("name").max_length)
@@ -909,9 +938,16 @@ class DuplicateTestCase(TestCase):
             entity=entity,
             collection=collection,
         )
+        collection.set_permission(Permission.EDIT, self.contributor)
 
         # Duplicate.
-        entities = Entity.objects.filter(id=entity.id).duplicate(self.contributor)
+
+        task = Entity.objects.filter(id=entity.id).duplicate(self.contributor)
+
+        entities = Entity.objects.filter(
+            pk__in=task.result(final_statuses=["OK", "ER"])
+        )
+
         self.assertEqual(len(entities), 1)
         duplicate = entities[0]
 
@@ -937,10 +973,8 @@ class DuplicateTestCase(TestCase):
             "name",
             "duplicated",
             "modified",
-            "collection_id",
             "entity_id",
             "search",
-            "permission_group_id",
         )
         for model_dict in (
             entity_dict,
@@ -963,19 +997,19 @@ class DuplicateTestCase(TestCase):
         self.assertEqual(duplicate.contributor.username, "contributor")
         self.assertAlmostEqual(duplicate.duplicated, now(), delta=timedelta(seconds=3))
         self.assertAlmostEqual(duplicate.modified, now(), delta=timedelta(seconds=3))
-        self.assertEqual(duplicate.collection, None)
 
-        # Assert collection is not altered.
+        # Assert collection is altered.
         self.assertEqual(Collection.objects.count(), 1)
-        self.assertEqual(collection.entity_set.count(), 1)
-        self.assertEqual(collection.entity_set.first().name, "Entity")
-        self.assertEqual(collection.data.count(), 2)
+        self.assertEqual(collection.entity_set.count(), 2)
+        self.assertEqual(collection.data.count(), 4)
         collection_data = collection.data.all().order_by("name")
-        self.assertEqual(collection_data[0].name, "Data 1")
-        self.assertEqual(collection_data[1].name, "Data 2")
+        self.assertEqual(collection_data[0].name, "Copy of Data 1")
+        self.assertEqual(collection_data[1].name, "Copy of Data 2")
+        self.assertEqual(collection_data[2].name, "Data 1")
+        self.assertEqual(collection_data[3].name, "Data 2")
 
         # Assert permissions.
-        self.assertEqual(len(duplicate.get_permissions(self.contributor)), 4)
+        self.assertEqual(len(duplicate.get_permissions(self.contributor)), 2)
 
     def test_entity_duplicate_inherit(self):
         process = Process.objects.create(contributor=self.user)
@@ -993,44 +1027,12 @@ class DuplicateTestCase(TestCase):
         collection.set_permission(Permission.VIEW, self.contributor)
 
         # Duplicate.
-        duplicated_entity1 = Entity.objects.filter(id=entity.id).duplicate(
-            self.user, inherit_collection=False
-        )[0]
-
-        self.assertEqual(collection.entity_set.count(), 1)
-        self.assertEqual(collection.entity_set.first().id, entity.id)
-        self.assertEqual(collection.data.count(), 1)
-        self.assertEqual(collection.data.first().id, data.id)
-
-        # Assert permissions.
-        self.assertEqual(duplicated_entity1.data.count(), 1)
-        self.assertCountEqual(
-            collection.get_permissions(self.user), [Permission.VIEW, Permission.EDIT]
-        )
-        self.assertEqual(len(duplicated_entity1.get_permissions(self.user)), 4)
-        self.assertEqual(
-            len(duplicated_entity1.data.first().get_permissions(self.user)), 4
-        )
-        self.assertListEqual(
-            collection.get_permissions(self.contributor), [Permission.VIEW]
-        )
-        self.assertListEqual(
-            duplicated_entity1.get_permissions(self.contributor), [Permission.VIEW]
-        )
-        self.assertListEqual(
-            duplicated_entity1.data.first().get_permissions(self.contributor),
-            [Permission.VIEW],
-        )
-
-        with self.assertRaises(ValidationError):
-            Entity.objects.filter(id=entity.id).duplicate(
-                self.contributor, inherit_collection=True
-            )[0]
-
-        duplicated_entity2 = Entity.objects.filter(id=entity.id).duplicate(
-            self.user, inherit_collection=True
-        )[0]
+        task = Entity.objects.filter(id=entity.id).duplicate(self.contributor)
+        task.wait(final_statuses=[BackgroundTask.STATUS_ERROR])
+        self.assertCountEqual(task.output, ["User doesn't have 'edit' permission on ."])
 
+        task = Entity.objects.filter(id=entity.id).duplicate(self.user)
+        duplicated_entity2 = Entity.objects.get(pk__in=task.result())
         self.assertEqual(collection.entity_set.count(), 2)
         self.assertEqual(collection.entity_set.first().id, entity.id)
         self.assertEqual(collection.entity_set.last().id, duplicated_entity2.id)
@@ -1056,17 +1058,24 @@ class DuplicateTestCase(TestCase):
         )
 
     def test_collection_duplicate(self):
+        process = Process.objects.create(contributor=self.user)
         collection = Collection.objects.create(name="Collection", contributor=self.user)
         entity = Entity.objects.create(
             name="Entity", contributor=self.user, collection=collection
         )
+        data = Data.objects.create(
+            contributor=self.user,
+            process=process,
+            status=Data.STATUS_DONE,
+            entity=entity,
+            collection=collection,
+        )
 
         collection.set_permission(Permission.OWNER, self.contributor)
 
         # Duplicate.
-        collections = Collection.objects.filter(id=collection.id).duplicate(
-            self.contributor
-        )
+        task = Collection.objects.filter(id=collection.id).duplicate(self.contributor)
+        collections = Collection.objects.filter(pk__in=task.result())
         self.assertEqual(len(collections), 1)
         duplicate = collections[0]
 
@@ -1074,13 +1083,20 @@ class DuplicateTestCase(TestCase):
         duplicate_dict = Collection.objects.filter(id=duplicate.id).values()[0]
 
         entity_dict = Entity.objects.filter(id=entity.id).values()[0]
-        entity_duplicate = duplicate.entity_set.first()
+        entity_duplicate = duplicate.entity_set.get()
         entity_duplicate_dict = Entity.objects.filter(id=entity_duplicate.id).values()[
             0
         ]
 
+        data_dict = Data.objects.filter(id=data.id).values()[0]
+        data_duplicate = duplicate.data.get()
+        data_duplicate_dict = Data.objects.filter(id=data_duplicate.id).values()[0]
+
         self.assertTrue(duplicate.is_duplicate())
         self.assertTrue(entity_duplicate.is_duplicate())
+        self.assertTrue(data_duplicate.is_duplicate())
+
+        self.maxDiff = None
 
         # Pop fields that should differ and assert the remaining.
         fields_to_differ = (
@@ -1090,6 +1106,7 @@ class DuplicateTestCase(TestCase):
             "name",
             "duplicated",
             "modified",
+            "entity_id",
             "collection_id",
             "search",
             "permission_group_id",
@@ -1098,13 +1115,16 @@ class DuplicateTestCase(TestCase):
             collection_dict,
             duplicate_dict,
             entity_dict,
+            data_dict,
             entity_duplicate_dict,
+            data_duplicate_dict,
         ):
             for field in fields_to_differ:
                 model_dict.pop(field, "")
 
         self.assertDictEqual(collection_dict, duplicate_dict)
         self.assertDictEqual(entity_dict, entity_duplicate_dict)
+        self.assertDictEqual(data_dict, data_duplicate_dict)
 
         # Assert fields that differ.
         self.assertEqual(duplicate.slug, "copy-of-collection")
