@@ -160,12 +160,12 @@ class TestRelationsAPI(TransactionResolweAPITestCase):
         with CaptureQueriesContext(conn) as captured_queries:
             response = self._get_list(self.contributor)
             self.assertEqual(len(response.data), 10)
-            self.assertEqual(len(captured_queries), 8)
+            self.assertEqual(len(captured_queries), 6)
 
     def test_get(self):
         resp = self._get_detail(self.relation_group.pk, user=self.contributor)
         self.assertEqual(resp.status_code, status.HTTP_200_OK)
-
+        self.assertEqual(resp.data["slug"], "relation")
         self.assertEqual(resp.data["collection"]["id"], self.collection.pk)
         self.assertEqual(resp.data["type"], "group")
         self.assertEqual(resp.data["category"], "replicates")
@@ -461,10 +461,10 @@ class TestRelationsAPI(TransactionResolweAPITestCase):
         }
 
         resp = self._post(data, user=self.contributor)
-        self.assertEqual(resp.status_code, status.HTTP_400_BAD_REQUEST)
-        self.assertEqual(
-            resp.data["non_field_errors"],
-            ["The fields collection, category must make a unique set."],
+        self.assertEqual(resp.status_code, status.HTTP_409_CONFLICT)
+        self.assertIn(
+            "duplicate key value violates unique constraint",
+            resp.data["error"],
         )
 
     def test_create_missing_collection(self):
@@ -489,6 +489,14 @@ class TestRelationsAPI(TransactionResolweAPITestCase):
         # Make sure partitions were not deleted.
         self.assertEqual(self.relation_group.relationpartition_set.count(), 2)
 
+        # Change the relation slug.
+        new_slug = "my-unique-relation-slug"
+        data = {"slug": new_slug}
+        resp = self._patch(self.relation_group.pk, data, user=self.contributor)
+        self.assertEqual(resp.status_code, status.HTTP_200_OK)
+        self.relation_group.refresh_from_db()
+        self.assertEqual(self.relation_group.slug, new_slug)
+
         # Relation type cannot be changed
         data = {"type": "series"}
         resp = self._patch(self.relation_group.pk, data, user=self.contributor)
