@@ -1,12 +1,22 @@
 """Data viewset."""
+
+from drf_spectacular.utils import extend_schema
+
 from django.db.models import Prefetch
 
-from rest_framework import exceptions, mixins, viewsets
+from rest_framework import (
+    exceptions,
+    mixins,
+    permissions,
+    serializers,
+    status,
+    viewsets,
+)
 from rest_framework.decorators import action
 from rest_framework.response import Response
 
 from resolwe.flow.filters import DataFilter
-from resolwe.flow.models import Collection, Data, DescriptorSchema, Entity, Process
+from resolwe.flow.models import Data, DescriptorSchema, Process
 from resolwe.flow.models.utils import fill_with_defaults
 from resolwe.flow.serializers import DataSerializer
 from resolwe.flow.utils import get_data_checksum
@@ -15,8 +25,11 @@ from resolwe.permissions.loader import get_permissions_class
 from resolwe.permissions.mixins import ResolwePermissionsMixin
 from resolwe.permissions.models import Permission, PermissionModel
 
+from .collection import BaseCollectionViewSet
+from .entity import EntityViewSet
 from .mixins import (
-    ParametersMixin,
+    ResolweBackgroundDeleteMixin,
+    ResolweBackgroundDuplicateMixin,
     ResolweCheckSlugMixin,
     ResolweCreateModelMixin,
     ResolweUpdateModelMixin,
@@ -24,55 +37,57 @@ from .mixins import (
 from .utils import get_collection_for_user
 
 
+class IsStaffuser(permissions.BasePermission):
+    """Allow access only to staff users."""
+
+    message = "Only staff users are allowed."
+
+    def has_permission(self, request, view):
+        """Return true when request is allowed."""
+        return bool(
+            request.user and request.user.is_authenticated and request.user.is_staff
+        )
+
+
+class MoveDataToCollectionSerializer(serializers.Serializer):
+    """Deserializer for data move to collection endpoint."""
+
+    ids = serializers.ListField(child=serializers.IntegerField())
+    destination_collection = serializers.IntegerField()
+
+
+class RestartSerializer(serializers.Serializer):
+    """Serializer for restarting a Data object."""
+
+    resource_overrides = serializers.JSONField(required=False, default=dict)
+
+
 class DataViewSet(
     ObservableMixin,
     ResolweCreateModelMixin,
     mixins.ListModelMixin,
     mixins.RetrieveModelMixin,
     ResolweUpdateModelMixin,
-    mixins.DestroyModelMixin,
+    ResolweBackgroundDeleteMixin,
     ResolwePermissionsMixin,
     ResolweCheckSlugMixin,
-    ParametersMixin,
+    ResolweBackgroundDuplicateMixin,
     viewsets.GenericViewSet,
 ):
     """API view for :class:`Data` objects."""
 
     qs_permission_model = PermissionModel.objects.select_related("user", "group")
-    qs_collection_ds = DescriptorSchema.objects.select_related("contributor")
-    qs_collection = Collection.objects.select_related("contributor")
-    qs_collection = qs_collection.prefetch_related(
-        "data",
-        "entity_set",
-        Prefetch("descriptor_schema", queryset=qs_collection_ds),
-    )
-
     qs_descriptor_schema = DescriptorSchema.objects.select_related("contributor")
-
-    qs_entity_col_ds = DescriptorSchema.objects.select_related("contributor")
-    qs_entity_col = Collection.objects.select_related("contributor")
-    qs_entity_col = qs_entity_col.prefetch_related(
-        "data",
-        "entity_set",
-        Prefetch("descriptor_schema", queryset=qs_entity_col_ds),
-    )
-    qs_entity_ds = DescriptorSchema.objects.select_related("contributor")
-    qs_entity = Entity.objects.select_related("contributor")
-    qs_entity = qs_entity.prefetch_related(
-        "data",
-        Prefetch("collection", queryset=qs_entity_col),
-        Prefetch("descriptor_schema", queryset=qs_entity_ds),
-    )
     qs_process = Process.objects.select_related("contributor")
     queryset = Data.objects.select_related("contributor").prefetch_related(
-        Prefetch("collection", queryset=qs_collection),
+        Prefetch("collection", queryset=BaseCollectionViewSet.queryset),
         Prefetch("descriptor_schema", queryset=qs_descriptor_schema),
-        Prefetch("entity", queryset=qs_entity),
+        Prefetch("entity", queryset=EntityViewSet.queryset),
         Prefetch("process", queryset=qs_process),
     )
 
     serializer_class = DataSerializer
-    filter_class = DataFilter
+    filterset_class = DataFilter
     permission_classes = (get_permissions_class(),)
 
     ordering_fields = (
@@ -95,34 +110,6 @@ class DataViewSet(
         """Prefetch permissions for current user."""
         return self.prefetch_current_user_permissions(self.queryset)
 
-    @action(detail=False, methods=["post"])
-    def duplicate(self, request, *args, **kwargs):
-        """Duplicate (make copy of) ``Data`` objects."""
-        if not request.user.is_authenticated:
-            raise exceptions.NotFound
-
-        inherit_collection = request.data.get("inherit_collection", False)
-        ids = self.get_ids(request.data)
-        queryset = Data.objects.filter(id__in=ids).filter_for_user(
-            request.user, Permission.VIEW
-        )
-        actual_ids = queryset.values_list("id", flat=True)
-        missing_ids = list(set(ids) - set(actual_ids))
-        if missing_ids:
-            raise exceptions.ParseError(
-                "Data objects with the following ids not found: {}".format(
-                    ", ".join(map(str, missing_ids))
-                )
-            )
-
-        duplicated = queryset.duplicate(
-            contributor=request.user,
-            inherit_collection=inherit_collection,
-        )
-
-        serializer = self.get_serializer(duplicated, many=True)
-        return Response(serializer.data)
-
     @action(detail=False, methods=["post"])
     def get_or_create(self, request, *args, **kwargs):
         """Get ``Data`` object if similar already exists, otherwise create it."""
@@ -167,21 +154,47 @@ class DataViewSet(
         serializer = self.get_serializer(queryset, many=True)
         return Response(serializer.data)
 
+    @extend_schema(
+        filters=False, responses={status.HTTP_200_OK: DataSerializer(many=True)}
+    )
     @action(detail=True)
     def parents(self, request, pk=None):
         """Return parents of the current data object."""
         return self._parents_children(request, self.get_object().parents)
 
+    @extend_schema(
+        filters=False,
+        request=RestartSerializer(),
+        responses={status.HTTP_200_OK: DataSerializer(many=True)},
+    )
+    @action(detail=True, methods=["post"], permission_classes=[IsStaffuser])
+    def restart(self, request, pk=None):
+        """Restart the current data object."""
+        argument_validator = RestartSerializer(data=request.data)
+        argument_validator.is_valid(raise_exception=True)
+        data = self.get_object()
+        try:
+            data.restart(**argument_validator.validated_data)
+        except RuntimeError as e:
+            raise exceptions.ValidationError(str(e))
+        return Response(self.get_serializer(data).data)
+
     @action(detail=True)
     def children(self, request, pk=None):
         """Return children of the current data object."""
         return self._parents_children(request, self.get_object().children.all())
 
+    @extend_schema(
+        request=MoveDataToCollectionSerializer(),
+        responses={status.HTTP_200_OK: None},
+    )
     @action(detail=False, methods=["post"])
     def move_to_collection(self, request, *args, **kwargs):
         """Move data objects to destination collection."""
-        ids = self.get_ids(request.data)
-        dst_collection_id = self.get_id(request.data, "destination_collection")
+        serializer = MoveDataToCollectionSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        ids = serializer.validated_data["ids"]
+        dst_collection_id = serializer.validated_data["destination_collection"]
 
         dst_collection = get_collection_for_user(dst_collection_id, request.user)
 
