@@ -1,17 +1,33 @@
 """Mixins used in Resolwe Viewsets."""
+
+from drf_spectacular.utils import extend_schema
+
 from django.db import IntegrityError, transaction
 
-from rest_framework import mixins, status
+from rest_framework import exceptions, mixins, serializers, status
 from rest_framework.decorators import action
-from rest_framework.exceptions import ParseError
 from rest_framework.generics import get_object_or_404
+from rest_framework.permissions import IsAuthenticated
 from rest_framework.response import Response
 
-from resolwe.observers.protocol import post_permission_changed
-from resolwe.permissions.models import get_anonymous_user
+from resolwe.observers.models import BackgroundTask
+from resolwe.observers.views import BackgroundTaskSerializer
+from resolwe.permissions.models import Permission, get_anonymous_user
 from resolwe.permissions.utils import assign_contributor_permissions
 
 
+class DuplicateSerializer(serializers.Serializer):
+    """Deserializer for duplicate endpoint."""
+
+    ids = serializers.ListField(child=serializers.IntegerField(), allow_empty=False)
+
+
+class DeleteSerializer(serializers.Serializer):
+    """Bulk delete given objects."""
+
+    ids = serializers.ListField(child=serializers.IntegerField(), allow_empty=False)
+
+
 class ResolweCreateModelMixin(mixins.CreateModelMixin):
     """Mixin to support creating new `Resolwe` models.
 
@@ -51,12 +67,116 @@ class ResolweCreateModelMixin(mixins.CreateModelMixin):
                 # in container.
                 if not instance.in_container():
                     assign_contributor_permissions(instance)
-                # The object was created inheriting permissions from its container.
-                # Notify observers.
-                else:
-                    post_permission_changed.send(
-                        sender=type(instance), instance=instance
-                    )
+
+
+class ResolweBackgroundDuplicateMixin:
+    """Duplicate objects in the background."""
+
+    # The method background queryset must have.
+    BACKGROUND_DUPLICATE_METHOD = "duplicate"
+
+    @extend_schema(
+        filters=False,
+        request=DuplicateSerializer(),
+        responses={200: BackgroundTaskSerializer},
+    )
+    @action(detail=False, methods=["post"], permission_classes=[IsAuthenticated])
+    def duplicate(self, request, *args, **kwargs):
+        """Bulk duplicate  objects in the background.
+
+        The background task is returned.
+        """
+        serializer = DuplicateSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        Model = self.serializer_class.Meta.model
+
+        # First check if the queryset supports background duplication.
+        if not hasattr(Model.objects.none(), self.BACKGROUND_DUPLICATE_METHOD):
+            raise exceptions.ValidationError(
+                f"The model '{Model}' does not support background duplication."
+            )
+
+        ids = serializer.validated_data["ids"]
+        to_duplicate = Model.objects.filter(id__in=ids).filter_for_user(request.user)
+        actual_ids = to_duplicate.values_list("id", flat=True)
+        missing_ids = list(set(ids) - set(actual_ids))
+        if missing_ids:
+            id_string = ", ".join(map(str, missing_ids))
+            raise exceptions.PermissionDenied(
+                f"Objects with ids {id_string} not found."
+            )
+
+        duplicate_method = getattr(to_duplicate, self.BACKGROUND_DUPLICATE_METHOD)
+        task = duplicate_method(contributor=request.user)
+
+        return Response(
+            status=status.HTTP_200_OK,
+            data=BackgroundTaskSerializer(task).data,
+        )
+
+
+class ResolweBackgroundDeleteMixin(mixins.DestroyModelMixin):
+    """Delete a model instance in the background if possible."""
+
+    BACKGROUND_DELETE_METHOD = "delete_background"
+
+    def perform_destroy(self, instance):
+        """Perform the actual delete.
+
+        If possible, delete the object in the background.
+        """
+        return getattr(instance, self.BACKGROUND_DELETE_METHOD, "delete")()
+
+    def destroy(self, request, *args, **kwargs):
+        """Delete a resource."""
+        instance = self.get_object()
+        return_value = self.perform_destroy(instance)
+        # When the object is deleted in the background, return a task description.
+        if isinstance(return_value, BackgroundTask):
+            return Response(
+                status=status.HTTP_200_OK,
+                data=BackgroundTaskSerializer(return_value).data,
+            )
+        else:
+            return Response(status=status.HTTP_204_NO_CONTENT)
+
+    @extend_schema(
+        filters=False,
+        request=DeleteSerializer(),
+        responses={200: BackgroundTaskSerializer},
+    )
+    @action(detail=False, methods=["post"])
+    def bulk_delete(self, request, *args, **kwargs):
+        """Bulk delete  objects in the background.
+
+        The background task is returned.
+        """
+        serializer = DeleteSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        ids = set(serializer.validated_data["ids"])
+        queryset = self.get_queryset()
+        if not getattr(queryset, self.BACKGROUND_DELETE_METHOD, None):
+            raise exceptions.ValidationError(
+                f"Model {queryset.model} does not support background deletion."
+            )
+
+        can_view = queryset.filter(id__in=ids).filter_for_user(
+            request.user, Permission.VIEW
+        )
+        can_delete = can_view.filter_for_user(request.user, Permission.EDIT)
+        can_view_ids = set(can_view.values_list("id", flat=True))
+        can_delete_ids = set(can_delete.values_list("id", flat=True))
+
+        if missing_ids := can_view_ids - can_delete_ids:
+            joined_ids = ", ".join(map(str, missing_ids))
+            raise exceptions.PermissionDenied(
+                f"No permission to delete objects with ids {joined_ids}."
+            )
+        task = getattr(can_delete, self.BACKGROUND_DELETE_METHOD, None)(request.user)
+        return Response(
+            status=status.HTTP_200_OK,
+            data=BackgroundTaskSerializer(task).data,
+        )
 
 
 class ResolweUpdateModelMixin(mixins.UpdateModelMixin):
@@ -123,9 +243,16 @@ class ResolweUpdateModelMixin(mixins.UpdateModelMixin):
         return obj
 
 
+class NameSerializer(serializers.Serializer):
+    """Serializer for the name field."""
+
+    name = serializers.CharField()
+
+
 class ResolweCheckSlugMixin:
     """Slug validation."""
 
+    @extend_schema(parameters=[NameSerializer()], responses={status.HTTP_200_OK: bool})
     @action(detail=False, methods=["get"])
     def slug_exists(self, request):
         """Check if given url slug exists.
@@ -137,46 +264,9 @@ class ResolweCheckSlugMixin:
         if not request.user.is_authenticated:
             return Response(status=status.HTTP_401_UNAUTHORIZED)
 
-        if "name" not in request.query_params:
-            return Response(
-                {"error": "Query parameter `name` must be given."},
-                status=status.HTTP_400_BAD_REQUEST,
-            )
+        serializer = NameSerializer(data=request.query_params)
+        serializer.is_valid(raise_exception=True)
+        slug_name = serializer.validated_data["name"]
 
         queryset = self.get_queryset()
-        slug_name = request.query_params["name"]
         return Response(queryset.filter(slug__iexact=slug_name).exists())
-
-
-class ParametersMixin:
-    """Mixin for viewsets for handling various parameters."""
-
-    def get_ids(self, request_data, parameter_name="ids"):
-        """Extract a list of integers from request data."""
-        if parameter_name not in request_data:
-            raise ParseError("`{}` parameter is required".format(parameter_name))
-
-        ids = request_data.get(parameter_name)
-        if not isinstance(ids, list):
-            raise ParseError("`{}` parameter not a list".format(parameter_name))
-
-        if not ids:
-            raise ParseError("`{}` parameter is empty".format(parameter_name))
-
-        if any(map(lambda id: not isinstance(id, int), ids)):
-            raise ParseError(
-                "`{}` parameter contains non-integers".format(parameter_name)
-            )
-
-        return ids
-
-    def get_id(self, request_data, parameter_name="id"):
-        """Extract an integer from request data."""
-        if parameter_name not in request_data:
-            raise ParseError("`{}` parameter is required".format(parameter_name))
-
-        id_parameter = request_data.get(parameter_name, None)
-        if not isinstance(id_parameter, int):
-            raise ParseError("`{}` parameter not an integer".format(parameter_name))
-
-        return id_parameter
