@@ -1,4 +1,5 @@
 """Relation viewset."""
+
 from itertools import zip_longest
 
 from django.db.models import Prefetch
@@ -7,9 +8,11 @@ from rest_framework import exceptions, permissions, status, viewsets
 from rest_framework.response import Response
 
 from resolwe.flow.filters import RelationFilter
-from resolwe.flow.models import Collection, DescriptorSchema, Relation
+from resolwe.flow.models import DescriptorSchema, Relation
 from resolwe.flow.serializers import RelationSerializer
+from resolwe.permissions.models import Permission
 
+from .collection import BaseCollectionViewSet
 from .mixins import ResolweCreateModelMixin
 
 
@@ -17,33 +20,30 @@ class RelationViewSet(ResolweCreateModelMixin, viewsets.ModelViewSet):
     """API view for :class:`Relation` objects."""
 
     qs_collection_ds = DescriptorSchema.objects.select_related("contributor")
-    qs_collection = Collection.objects.select_related("contributor")
-    qs_collection = qs_collection.prefetch_related(
-        "data",
-        "entity_set",
-        Prefetch("descriptor_schema", queryset=qs_collection_ds),
-    )
 
     queryset = (
         Relation.objects.all()
         .select_related("contributor", "type")
         .prefetch_related(
-            Prefetch("collection", queryset=qs_collection), "relationpartition_set"
+            Prefetch("collection", queryset=BaseCollectionViewSet.queryset),
+            "relationpartition_set",
         )
     )
+
     serializer_class = RelationSerializer
     permission_classes = (permissions.AllowAny,)
     filterset_class = RelationFilter
     ordering_fields = ("id", "created", "modified")
     ordering = ("id",)
 
-    def _filter_queryset(self, queryset):
+    def get_queryset(self):
         """Filter queryset by entity, label and position.
 
         Due to a bug in django-filter these filters have to be applied
         manually:
         https://github.com/carltongibson/django-filter/issues/883
         """
+        queryset = self.queryset
         entities = self.request.query_params.getlist("entity")
         labels = self.request.query_params.getlist("label")
         positions = self.request.query_params.getlist("position")
@@ -78,10 +78,10 @@ class RelationViewSet(ResolweCreateModelMixin, viewsets.ModelViewSet):
         If user has no view permission it returns the HTTP status code that
         shoud be sent back to the user.
         """
-        if not user.has_perm("view_collection", collection):
+        if not user.has_perm(Permission.VIEW, collection):
             return status.HTTP_404_NOT_FOUND
 
-        elif not user.has_perm("edit_collection", collection):
+        elif not user.has_perm(Permission.EDIT, collection):
             return (
                 status.HTTP_403_FORBIDDEN
                 if user.is_authenticated
@@ -90,10 +90,6 @@ class RelationViewSet(ResolweCreateModelMixin, viewsets.ModelViewSet):
         else:
             return True
 
-    def get_queryset(self):
-        """Get queryset and perform custom filtering."""
-        return self._filter_queryset(self.queryset)
-
     def create(self, request, *args, **kwargs):
         """Create a resource."""
         user = request.user
