@@ -1,4 +1,8 @@
 """Permissions functions used in Resolwe Viewsets."""
+
+import versionfield.fields
+from drf_spectacular.utils import extend_schema
+
 from django.conf import settings
 from django.db import models, transaction
 
@@ -7,6 +11,7 @@ from rest_framework.decorators import action
 from rest_framework.request import Request
 from rest_framework.response import Response
 
+from resolwe.auditlog.auditmanager import AuditManager
 from resolwe.permissions.models import Permission
 from resolwe.permissions.shortcuts import get_object_perms
 
@@ -14,7 +19,6 @@ from .utils import (
     check_owner_permission,
     check_public_permissions,
     check_user_permissions,
-    translate_from_old_syntax,
     update_permission,
 )
 
@@ -22,10 +26,32 @@ from .utils import (
 class CurrentUserPermissionsSerializer(serializers.Serializer):
     """Current user permissions serializer."""
 
-    id = serializers.IntegerField()
-    type = serializers.CharField(max_length=50)
-    name = serializers.CharField(max_length=100)
-    permissions = serializers.ListField(child=serializers.CharField(max_length=30))
+    id = serializers.IntegerField(required=False)
+    type = serializers.ChoiceField(choices=["user", "group", "public"])
+    name = serializers.CharField(max_length=100, required=False)
+    username = serializers.CharField(max_length=150, required=False)
+    permissions = serializers.ListField(
+        child=serializers.ChoiceField(choices=list(map(str, Permission)))
+    )
+
+
+class PermissionSerializer(serializers.Serializer):
+    """Permission serializer.
+
+    Dictionary keys for users and groups are either ids or usernames (emails) or group names.
+    """
+
+    permission_choices = [Permission(i).name for i in list(Permission.highest())]
+
+    users = serializers.DictField(
+        child=serializers.ChoiceField(choices=permission_choices),
+        required=False,
+    )
+    groups = serializers.DictField(
+        child=serializers.ChoiceField(choices=permission_choices),
+        required=False,
+    )
+    public = serializers.ChoiceField(choices=permission_choices, required=False)
 
 
 class ResolwePermissionsMixin:
@@ -51,16 +77,29 @@ class ResolwePermissionsMixin:
         Include permissions information with objects.
 
         """
+        if not hasattr(type(self), "_permission_augmented_serializer_class"):
+            augmented_serializer = self._augment_serializer_class()
+            type(self)._permission_augmented_serializer_class = augmented_serializer
+
+        return type(self)._permission_augmented_serializer_class
+
+    def _augment_serializer_class(self):
+        """Create an augmented serializer class."""
         base_class = super().get_serializer_class()
 
         class SerializerWithPermissions(base_class):
-            """Augment serializer class."""
+            """Augmented serializer."""
+
+            serializer_field_mapping = {
+                **base_class.serializer_field_mapping,
+                versionfield.fields.VersionField: serializers.CharField,
+            }
 
             def get_fields(serializer_self):
                 """Return serializer's fields."""
                 fields = super().get_fields()
                 fields["current_user_permissions"] = CurrentUserPermissionsSerializer(
-                    read_only=True
+                    read_only=True, many=True
                 )
                 return fields
 
@@ -68,16 +107,29 @@ class ResolwePermissionsMixin:
                 """Object serializer."""
                 data = super().to_representation(instance)
                 if (
-                    "fields" not in self.request.query_params
-                    or "current_user_permissions" in self.request.query_params["fields"]
+                    "fields" not in serializer_self.request.query_params
+                    or "current_user_permissions"
+                    in serializer_self.request.query_params["fields"]
                 ):
-                    data["current_user_permissions"] = get_object_perms(
-                        instance, self.request.user, True
+                    to_serialize = get_object_perms(
+                        instance, serializer_self.request.user, True
                     )
+                    data["current_user_permissions"] = CurrentUserPermissionsSerializer(
+                        to_serialize, read_only=True, many=True
+                    ).data
+
                 return data
 
+        SerializerWithPermissions.__name__ = base_class.__name__ + "WithPermissions"
+        SerializerWithPermissions.__doc__ = base_class.__doc__
+
         return SerializerWithPermissions
 
+    @extend_schema(
+        request=PermissionSerializer(),
+        responses={status.HTTP_200_OK: CurrentUserPermissionsSerializer()},
+        filters=False,
+    )
     @action(
         detail=True,
         methods=["get", "post"],
@@ -98,32 +150,24 @@ class ResolwePermissionsMixin:
         if obj.has_permission(Permission.SHARE, request.user):
             obj.refresh_from_db()
 
+        audit_manager = AuditManager.global_instance()
+
         if request.method == "POST":
-            payload = translate_from_old_syntax(request.data, obj)
-            user = request.user
-            allow_owner = obj.is_owner(user) or user.is_superuser
-            check_owner_permission(payload, allow_owner, obj)
-            check_public_permissions(payload)
-            check_user_permissions(payload, request.user.pk)
+            audit_manager.log_message("Permissions updated: %s", request.data)
+            allow_owner = obj.is_owner(request.user) or request.user.is_superuser
+            check_owner_permission(request.data, allow_owner, obj)
+            check_public_permissions(request.data)
+            check_user_permissions(request.data, request.user.pk)
 
             with transaction.atomic():
-                update_permission(obj, payload)
+                update_permission(obj, request.data)
                 owner_count = obj.permission_group.permissions.filter(
                     value=Permission.OWNER.value, user__isnull=False
                 ).count()
 
                 if not owner_count:
                     raise exceptions.ParseError("Object must have at least one owner.")
+        else:
+            audit_manager.log_message("Permissions read: %s", request.data)
 
         return Response(get_object_perms(obj))
-
-    @action(
-        detail=False,
-        methods=["get", "post"],
-        url_path="permissions",
-        url_name="permissions",
-    )
-    def list_permissions(self, request: Request) -> Response:
-        """Batch get or set permissions API endpoint."""
-        # TODO: Implement batch get/set permissions
-        return Response(status=status.HTTP_501_NOT_IMPLEMENTED)
