@@ -1,8 +1,9 @@
 """Permission storage model."""
+
 import logging
 from enum import IntEnum
 from functools import reduce
-from typing import List, Optional, Union
+from typing import Iterable, List, Optional, Union
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
@@ -260,6 +261,18 @@ class PermissionGroup(models.Model):
             filter |= models.Q(is_superuser=True)
         return list(get_user_model().objects.filter(filter).distinct())
 
+    def groups_with_permission(self, permission: Permission) -> List[Group]:
+        """Get a list of groups with at least this permission level.
+
+        Calling this with Permission.NONE will return groups for which an explicit
+        PermissionModel with Permission.NONE exists.
+
+        :attr permission: the permission level group must have.
+        """
+        filtered_permissions = self.permissions.filter(value__gte=permission)
+        filter = models.Q(pk__in=filtered_permissions.values_list("group", flat=True))
+        return list(Group.objects.filter(filter).distinct())
+
 
 class PermissionQuerySet(models.QuerySet):
     """Queryset with methods that simlify filtering by permissions."""
@@ -463,6 +476,16 @@ class PermissionObject(models.Model):
         """
         return self.permission_group.users_with_permission(permission, with_superusers)
 
+    def groups_with_permission(self, permission: Permission) -> List[Group]:
+        """Get a list of groups with at least this permission level.
+
+        Calling this with Permission.NONE will return groups for which an explicit
+        PermissionModel with Permission.NONE exists.
+
+        :attr permission: the permission level group must have.
+        """
+        return self.permission_group.groups_with_permission(permission)
+
     @property
     def topmost_container(self) -> Optional[models.Model]:
         """Get the top-most container of the object.
@@ -475,6 +498,15 @@ class PermissionObject(models.Model):
                 return value
         return None
 
+    @property
+    def containers(self) -> Iterable[models.Model]:
+        """Get the sequence of containers this object is in.
+
+        :returns: a sequence of containers (may be empty) this object is in.
+        """
+        containers = (getattr(self, prop, None) for prop in self._container_properties)
+        return [container for container in containers if container is not None]
+
     def in_container(self) -> bool:
         """Return if object lies in a container."""
         return self.topmost_container is not None
