@@ -1,4 +1,5 @@
 """Process descriptor."""
+
 import collections
 import re
 
@@ -52,7 +53,7 @@ class ProcessDescriptor:
             """Return string representation."""
             return repr(self.__dict__)
 
-    def __init__(self, source=None):
+    def __init__(self, source=None, parent_process=None):
         """Construct a new process descriptor."""
         self.metadata = ProcessDescriptor.Metadata()
         self.inputs = collections.OrderedDict()
@@ -60,6 +61,7 @@ class ProcessDescriptor:
         self.relations = None
         self.name = None
         self.source = source
+        self.parent_process = parent_process
 
     def validate(self):
         """Validate process descriptor."""
@@ -94,7 +96,7 @@ class ProcessDescriptor:
             "requirements": {
                 "executor": {
                     "docker": {
-                        "image": "resolwe/base:ubuntu-18.04",
+                        "image": "public.ecr.aws/s4q6j6e8/resolwe/base:ubuntu-18.04",
                     },
                 },
             },
