@@ -1,4 +1,5 @@
 """Process input or output fields."""
+
 import collections
 import glob
 import gzip
@@ -16,10 +17,7 @@ from typing import Dict, Iterable, List, Optional, Set, Tuple, Type, Union
 import requests
 
 from .communicator import communicator
-
-DATA_LOCAL_VOLUME = Path(os.environ.get("DATA_LOCAL_VOLUME", "/data_local"))
-DATA_VOLUME = Path(os.environ.get("DATA_VOLUME", "/data"))
-DATA_ALL_VOLUME = Path(os.environ.get("DATA_ALL_VOLUME", "/data_all"))
+from .descriptor import ProcessDescriptor
 
 # Upload files in batches of 1000.
 UPLOAD_FILE_BATCH_SIZE = 1000
@@ -139,13 +137,14 @@ class Field:
         default=None,
         choices=None,
         allow_custom_choice=None,
+        disabled=False,
         hidden=False,
         *args,
         **kwargs,
     ):
         """Construct a field descriptor."""
         self.name = None
-        self.process: Optional["resolwe.process.descriptor.ProcessDescriptor"] = None
+        self.process: Optional[ProcessDescriptor] = None
         self.label = label
         self.required = required
         self.description = description
@@ -153,6 +152,7 @@ class Field:
         self.choices = choices
         self.allow_custom_choice = allow_custom_choice
         self.hidden = hidden
+        self.disabled = disabled
 
     @property
     def _descriptor_field_name(self):
@@ -207,6 +207,8 @@ class Field:
             schema["default"] = self.default
         if self.hidden is not None:
             schema["hidden"] = self.hidden
+        if self.disabled is not None:
+            schema["disabled"] = self.disabled
         if self.allow_custom_choice is not None:
             schema["allow_custom_choice"] = self.allow_custom_choice
         if self.choices is not None:
@@ -304,6 +306,18 @@ class IntegerField(Field):
 
     field_type = "basic:integer"
 
+    def __init__(self, range=None, *args, **kwargs):
+        """Construct an Integer field descriptor."""
+        self.range = range
+        super().__init__(*args, **kwargs)
+
+    def to_schema(self):
+        """Return Integer field schema."""
+        schema = super().to_schema()
+        if self.range is not None:
+            schema["range"] = self.range
+        return schema
+
     def to_python(self, value):
         """Convert value if needed."""
         if value is not None:
@@ -319,6 +333,18 @@ class FloatField(Field):
     # TODO: Fix the underlying field into basic:float once that is renamed.
     field_type = "basic:decimal"
 
+    def __init__(self, range=None, *args, **kwargs):
+        """Construct a Float field descriptor."""
+        self.range = range
+        super().__init__(*args, **kwargs)
+
+    def to_schema(self):
+        """Return Float field schema."""
+        schema = super().to_schema()
+        if self.range is not None:
+            schema["range"] = self.range
+        return schema
+
     def to_python(self, value):
         """Convert value if needed."""
         if value is not None:
@@ -377,7 +403,7 @@ class UrlField(Field):
                 raise ValidationError("field's url element must be a string")
 
             return value
-        elif not isinstance(value, None):
+        elif value is not None:
             raise ValidationError("field must be a string or a dict")
 
     def get_field_type(self):
@@ -599,9 +625,12 @@ class FileDescriptor:
                 else file_name
             )
 
-        # Large file download from Google Drive requires cookie and token.
         try:
             response = None
+            # Resolve the given URL.
+            src = communicator.resolve_url(src)
+
+            # Large file download from Google Drive requires cookie and token.
             if re.match(
                 r"^https://drive.google.com/[-A-Za-z0-9\+&@#/%?=~_|!:,.;]*[-A-Za-z0-9\+&@#/%=~_|]$",
                 src,
@@ -624,11 +653,12 @@ class FileDescriptor:
                 src,
             ):
                 response = requests.get(src, stream=True)
+
         except requests.exceptions.ConnectionError:
             raise requests.exceptions.ConnectionError(
                 "Could not connect to {}".format(src)
             )
-
+        # The response was received with OK status code.
         if response:
             with open(file_name, "wb") as f:
                 total = response.headers.get("content-length")
@@ -662,12 +692,25 @@ class FileDescriptor:
                 raise ValueError("Downloaded file not found {}".format(file_name))
 
             src = file_name
+        # The response was received with error code indicating failure.
+        elif response is not None:
+            response_content = next(response.iter_content(chunk_size=200))
+            raise RuntimeError(
+                f"Error when downoading {src}. The received response code is "
+                f"{response.status_code}, first 200 bytes of the response: "
+                f"'{response_content}'."
+            )
+        # The response is still None, import the file from upload directory.
         else:
             # If scr is file it needs to have upload directory prepended.
-            upload_dir = Path(os.environ.get("UPLOAD_DIR", "/upload"))
-            src_path = upload_dir / src
+            if "UPLOAD_DIR" not in os.environ:
+                raise RuntimeError(
+                    "No upload directory on filesystem is defined, "
+                    f"can not import file {src} from filesystem."
+                )
+            src_path = Path(os.environ["UPLOAD_DIR"]) / src
             if not src_path.is_file():
-                raise ValueError(f"Source file not found {src}")
+                raise ValueError(f"Source file not found {src_path}")
             src = os.fspath(src_path)
 
         # Decide which import should be used.
@@ -745,7 +788,7 @@ class FileField(Field):
                 refs=refs,
                 file_field=self,
             )
-        elif not isinstance(value, None):
+        elif value is not None:
             raise ValidationError("field must be a FileDescriptor, string or a dict")
 
     def to_output(self, value):
@@ -831,7 +874,7 @@ class DirField(Field):
                 raise ValidationError("field's refs element must be a list of strings")
 
             return DirDescriptor(path, size=size, total_size=total_size, refs=refs)
-        elif not isinstance(value, None):
+        elif value is not None:
             raise ValidationError("field must be a DirDescriptor, string or a dict")
 
     def to_output(self, value):
@@ -1195,11 +1238,15 @@ class GroupField(Field):
         """
         # Use order-preserving definition namespace (__dict__) to respect the
         # order of GroupField's fields definition.
+        # Skip private members and attributes that are not subclasses of the
+        # Field class.
         for field_name in self.field_group.__dict__:
             if field_name.startswith("_"):
                 continue
 
             field = getattr(self.field_group, field_name)
+            if not isinstance(field, Field):
+                continue
             field.contribute_to_class(process, self.fields, field_name)
 
         super().contribute_to_class(process, fields, name)
