@@ -1,4 +1,5 @@
 """Storage connector."""
+
 import abc
 import copy
 from enum import Enum, auto
@@ -315,6 +316,14 @@ class BaseStorageConnector(metaclass=abc.ABCMeta):
         """
         raise NotImplementedError
 
+    @validate_url
+    def url(self, url: Union[str, PathLike]) -> Optional[str]:
+        """Return the URL to the given file in connector native notation.
+
+        :returns: the url to the given file in connector native notation.
+        """
+        raise NotImplementedError
+
     def __eq__(self, other):
         """Equality check.
 
