@@ -1,8 +1,9 @@
 """Registry class for storage connectors."""
-import logging
-from typing import MutableMapping
 
-from .baseconnector import BaseStorageConnector
+import logging
+from typing import Dict, MutableMapping, Sequence, Type
+
+from .baseconnector import DEFAULT_CONNECTOR_PRIORITY, BaseStorageConnector
 
 logger = logging.getLogger(__name__)
 
@@ -18,16 +19,11 @@ class StorageConnectors(MutableMapping[str, BaseStorageConnector]):
     of it will always return the same instance.
     """
 
-    __instance = None  #  A single instance of this class
-
-    @classmethod
-    def __new__(cls, *args, **kwargs):
-        """Create or return singleton."""
-        if StorageConnectors.__instance is None:
-            StorageConnectors.__instance = super().__new__(cls)
-            StorageConnectors.__instance.__setattr__("_connectors", dict())
-            StorageConnectors.__instance.__setattr__("_connector_classes", dict())
-        return StorageConnectors.__instance
+    def __init__(self):
+        """Initialize."""
+        self._connectors: Dict[str, BaseStorageConnector] = dict()
+        self._connector_classes: Dict[str, Type[BaseStorageConnector]] = dict()
+        self._storage_connectors: Dict[str, Sequence[BaseStorageConnector]] = dict()
 
     def _get_connectors_settings(self) -> dict:
         """Return settings for storage connectors.
@@ -43,14 +39,42 @@ class StorageConnectors(MutableMapping[str, BaseStorageConnector]):
             settings = SETTINGS.get(key, {})
         except ImportError:
             # Import settings from Django.
-            from resolwe.storage.settings import STORAGE_CONNECTORS as settings
+            from resolwe.storage.settings import (
+                STORAGE_CONNECTORS as settings,  # type: ignore
+            )
 
         return settings
 
-    def add_storage_connector_class(self, connector: BaseStorageConnector):
+    def _recreate_storage_mappings(self):
+        """Recreate mapping between storages and connectors."""
+        try:
+            # Try to import settings from executor.
+            from executors.global_settings import SETTINGS
+
+            storages = SETTINGS.get("FLOW_STORAGE", dict())
+        except ImportError:
+            # Import settings from Django.
+            from resolwe.storage import settings as storage_settings
+
+            storages = storage_settings.FLOW_STORAGE
+
+        self._storage_connectors = dict()
+        for storage_name, storage_config in storages.items():
+            self._storage_connectors[storage_name] = sorted(
+                (
+                    self[connector_name]
+                    for connector_name in storage_config["connectors"]
+                    if connector_name in self
+                ),
+                key=lambda connector: connector.config.get(
+                    "priority", DEFAULT_CONNECTOR_PRIORITY
+                ),
+            )
+
+    def add_storage_connector_class(self, connector: Type[BaseStorageConnector]):
         """Add storage connector class to the registry."""
 
-        def _get_class_name(klass: BaseStorageConnector):
+        def _get_class_name(klass: Type[BaseStorageConnector]):
             return "{}.{}".format(klass.__module__, klass.__name__)
 
         assert issubclass(connector, BaseStorageConnector)
@@ -60,7 +84,7 @@ class StorageConnectors(MutableMapping[str, BaseStorageConnector]):
         self._add_connectors_from_settings(full_class_name, connector)
 
     def _add_connectors_from_settings(
-        self, full_class_name: str, klass: BaseStorageConnector
+        self, full_class_name: str, klass: Type[BaseStorageConnector]
     ):
         """Create instances for the connector of the given type.
 
@@ -80,6 +104,11 @@ class StorageConnectors(MutableMapping[str, BaseStorageConnector]):
                         )
                 instance = klass(connector_settings["config"], connector_name)
                 self[connector_name] = instance
+        self._recreate_storage_mappings()
+
+    def for_storage(self, storage: str) -> Sequence[BaseStorageConnector]:
+        """Get storage connectors for storage ordered by priority."""
+        return self._storage_connectors[storage]
 
     def recreate_connectors(self):
         """Recreate connectors from settings."""
