@@ -1,4 +1,5 @@
 """Data transfer between connectors."""
+
 import concurrent.futures
 import logging
 from contextlib import suppress
@@ -10,6 +11,7 @@ from typing import TYPE_CHECKING, Iterable, List, Union
 import wrapt
 from requests.exceptions import ConnectionError as RequestsConnectionError
 from requests.exceptions import ReadTimeout
+from urllib3.exceptions import ProtocolError
 
 from .baseconnector import BaseStorageConnector
 from .circular_buffer import CircularBuffer
@@ -41,22 +43,31 @@ ERROR_TIMEOUT = 5  # In seconds.
 transfer_exceptions = tuple(
     boto_exceptions
     + gcs_exceptions
+    + [RequestsConnectionError, ReadTimeout, ConnectionResetError, ProtocolError]
     + [DataTransferError]
-    + [RequestsConnectionError, ReadTimeout]
 )
 
 
 @wrapt.decorator
 def retry_on_transfer_error(wrapped, instance, args, kwargs):
     """Retry on tranfser error."""
-    for _ in range(ERROR_MAX_RETRIES):
+    for retry in range(1, ERROR_MAX_RETRIES + 1):
         try:
             return wrapped(*args, **kwargs)
-        except transfer_exceptions as err:
-            connection_err = err
-            sleep(ERROR_TIMEOUT)
-
-    raise connection_err
+        except transfer_exceptions:
+            # Log the exception on retry for inspection.
+            if retry != ERROR_MAX_RETRIES:
+                logger.exception(
+                    "Retry %d/%d got exception, will retry in %d seconds.",
+                    retry,
+                    ERROR_MAX_RETRIES,
+                    ERROR_TIMEOUT,
+                )
+                sleep(ERROR_TIMEOUT)
+            # Raise exception when max retries are exceeded.
+            else:
+                logger.exception("Final retry got exception, re-raising it.")
+                raise
 
 
 class Transfer:
@@ -266,13 +277,17 @@ class Transfer:
         #   from from_connector to to_connector.
         if from_connector.can_open_stream:
             stream = from_connector.open_stream(from_url, "rb")
-            to_connector.push(stream, to_base_url / to_url, chunk_size=chunk_size)
+            to_connector.push(
+                stream, to_base_url / to_url, chunk_size=chunk_size, hashes=hashes
+            )
             stream.close()
 
         elif to_connector.can_open_stream:
             stream = to_connector.open_stream(to_base_url / to_url, "wb")
             from_connector.get(from_url, stream, chunk_size=chunk_size)
             stream.close()
+            to_connector.set_hashes(to_base_url / to_url, hashes)
+
         # Otherwise create out own stream and use threads to transfer data.
         else:
 
@@ -299,6 +314,7 @@ class Transfer:
                     data_stream,
                     to_base_url / to_url,
                     chunk_size=chunk_size,
+                    hashes=hashes,
                 )
                 download_task.add_done_callback(partial(future_done, data_stream))
                 futures = (download_task, upload_task)
@@ -340,9 +356,6 @@ class Transfer:
                     f"{common_hash_type}: expected {from_hash}, got {to_hash}."
                 )
 
-        # Store computed hashes as metadata for later use.
-        to_connector.set_hashes(to_base_url / to_url, hashes)
-
         for hash_type in to_connector.refresh_hash_after_transfer:
             hash = to_connector.get_hash(to_base_url / to_url, hash_type)
             object_[hash_type] = hash
