@@ -1,5 +1,7 @@
 """Storage application views."""
+
 import json
+import logging
 import re
 import tempfile
 from datetime import datetime
@@ -12,18 +14,70 @@ from urllib.parse import urlencode, urlparse
 import pytz
 from bs4 import BeautifulSoup
 
-from django.core.exceptions import PermissionDenied
+from django.core.exceptions import ImproperlyConfigured, PermissionDenied
 from django.http import HttpRequest, HttpResponse, JsonResponse
 from django.shortcuts import redirect
 from django.urls import reverse
 from django.utils import timezone
+from django.utils.decorators import method_decorator
 from django.views import View
+from django.views.decorators.csrf import csrf_exempt
+
+from rest_framework import serializers
+from rest_framework.response import Response
+from rest_framework.viewsets import ViewSet
 
 from resolwe.flow.models import Data
 from resolwe.flow.models.collection import Collection
-from resolwe.permissions.shortcuts import get_objects_for_user
+from resolwe.permissions.utils import get_user
+from resolwe.storage.connectors import connectors
 from resolwe.storage.connectors.baseconnector import BaseStorageConnector
 from resolwe.storage.models import FileStorage, ReferencedPath
+from resolwe.test.utils import ignore_in_tests
+
+logger = logging.getLogger(__name__)
+
+
+class UploadConfigSerializer(serializers.Serializer):
+    """Serializer for upload configuration."""
+
+    type = serializers.CharField()
+    config = serializers.DictField()
+
+
+class UploadConfig(ViewSet):
+    """Get the upload configuration."""
+
+    serializer_class = UploadConfigSerializer
+
+    def list(self, request):
+        """Return the JSON representing the upload configuration.
+
+        The returning object is JSON representation of the dictionary with the
+        following fields:
+
+        - type: the type of upload connector. Currently we
+          support 'LOCAL', and 'S3' connector types. Upload through server is
+          always supported.
+
+        - credentials: the dictionary representing the set of credentials that
+          are used to upload data. This dictionary is specific to connector
+          type and may be empty.
+        """
+        try:
+            upload_connector = connectors.for_storage("upload")[0]
+            prefix = str(get_user(request.user).id)
+            response = {
+                "type": upload_connector.CONNECTOR_TYPE.name,
+                "config": upload_connector.temporary_credentials(prefix),
+            }
+        except Exception:
+            message = "Upload connector could not be determined."
+            logger.exception(message)
+            raise ImproperlyConfigured(message)
+
+        serializer = self.serializer_class(response)
+        return Response(serializer.data)
 
 
 class DataBrowseView(View):
@@ -54,6 +108,7 @@ class DataBrowseView(View):
             "type": "file",
             "mtime": self._get_mtime(referenced_path.file_storage.created),
             "size": referenced_path.size,
+            "md5": referenced_path.md5,
         }
         if referenced_path.path.endswith("/"):
             data.pop("size")
@@ -63,9 +118,7 @@ class DataBrowseView(View):
     def _get_datum(self, data_id: int) -> Data:
         """Get datum with given id and check it's permissions."""
         # Check permissions and load requested data object.
-        data = get_objects_for_user(
-            self.request.user, "view_data", Data.objects.filter(pk=data_id)
-        )
+        data = Data.objects.filter(pk=data_id).filter_for_user(self.request.user)
         # Send response with status 403 when requested data object does not exists.
         if not data.exists():
             raise PermissionDenied()
@@ -224,6 +277,7 @@ class HTMLPreprocessorView(DataBrowseView):
         return response
 
 
+@method_decorator(csrf_exempt, name="dispatch")
 class UriResolverView(DataBrowseView):
     """Get resolved and signed URLs for a given list of URIs."""
 
@@ -258,7 +312,10 @@ class UriResolverView(DataBrowseView):
             data_id = int(match.group(1))
             relative_path = Path(match.group(2))
             datum = self._get_datum(data_id)
-            response_data[uri] = self._get_response(datum, relative_path)[0]
+            try:
+                response_data[uri] = self._get_response(datum, relative_path)[0]
+            except PermissionDenied:
+                response_data[uri] = ""
 
         return JsonResponse(response_data)
 
@@ -271,14 +328,13 @@ class UriResolverView(DataBrowseView):
         check. However, if they are from the same collection it is easy
         to check, just confirm that the user has view permission on
         collection. That can be checked only once and than cached.
+        Except in tests, where colletion ids may be reused.
         """
 
-        @lru_cache(maxsize=1)
+        @ignore_in_tests(lru_cache(maxsize=1))
         def has_view_permission(collection_id: int) -> bool:
-            collection = get_objects_for_user(
-                self.request.user,
-                "view_collection",
-                Collection.objects.filter(pk=collection_id),
+            collection = Collection.objects.filter(pk=collection_id).filter_for_user(
+                self.request.user
             )
             return collection.exists()
 
