@@ -13,6 +13,7 @@ import io
 import json
 import os
 import shutil
+import sys
 import tarfile
 import tempfile
 import time
@@ -25,11 +26,11 @@ from django.core import management
 from django.core.exceptions import ValidationError
 from django.core.validators import URLValidator
 from django.db import transaction
-from django.utils.crypto import get_random_string
 from django.utils.text import slugify
 
-from resolwe.flow.models import Collection, Data, DescriptorSchema, Process, Storage
+from resolwe.flow.models import Data, DescriptorSchema, Process, Storage
 from resolwe.flow.utils import dict_dot, iterate_fields, iterate_schema
+from resolwe.permissions.models import PermissionGroup
 from resolwe.storage.connectors import connectors
 from resolwe.test import TransactionTestCase
 
@@ -138,6 +139,11 @@ class ProcessTestCase(TransactionTestCase):
         for schema in schemas:
             schema.contributor = self.admin
 
+    def _create_permission_groups(self, schemas):
+        """Create permission groups."""
+        for schema in schemas:
+            schema.permission_group = PermissionGroup.objects.create()
+
     def _register_schemas(
         self, path=None, processes_paths=None, descriptors_paths=None
     ):
@@ -208,6 +214,7 @@ class ProcessTestCase(TransactionTestCase):
                     SCHEMAS_FIXTURE_CACHE[cache_key][schemas["name"]]
                 )
                 self._update_schema_relations(schemas_cache)
+                self._create_permission_groups(schemas_cache)
                 schemas["model"].objects.bulk_create(schemas_cache)
         else:
             if processes_paths is None and descriptors_paths is None:
@@ -241,15 +248,6 @@ class ProcessTestCase(TransactionTestCase):
 
         return stdout, stderr
 
-    def _create_collection(self):
-        """Create a test collection for admin user.
-
-        :return: created test collection
-        :rtype: Collection
-
-        """
-        return Collection.objects.create(name="Test collection", contributor=self.admin)
-
     def setUp(self):
         """Initialize test data."""
         super().setUp()
@@ -259,7 +257,6 @@ class ProcessTestCase(TransactionTestCase):
         if stderr:
             raise RuntimeError(stderr)
 
-        self.collection = self._create_collection()
         upload_connectors = [
             connector
             for connector in connectors.for_storage("upload")
@@ -304,7 +301,10 @@ class ProcessTestCase(TransactionTestCase):
                 return exc_list[-1][1]
 
         result = self.defaultTestResult()
-        self._feedErrorsToResult(result, self._outcome.errors)
+
+        # Python 3.11 has no feedErrorsToResult method.
+        if sys.version_info.major == 3 and sys.version_info.minor <= 10:
+            self._feedErrorsToResult(result, self._outcome.errors)
         error = list2reason(result.errors)
         failure = list2reason(result.failures)
 
@@ -367,6 +367,8 @@ class ProcessTestCase(TransactionTestCase):
         verbosity=0,
         tags=None,
         contributor=None,
+        collection=None,
+        process_resources=None,
     ):
         """Run the specified process with the given inputs.
 
@@ -501,11 +503,11 @@ class ProcessTestCase(TransactionTestCase):
             input=input_,
             contributor=contributor or self.admin,
             process=process,
-            slug=get_random_string(length=6),
             tags=tags or [],
             descriptor_schema=descriptor_schema,
             descriptor=descriptor or {},
-            collection=self.collection,
+            collection=collection,
+            process_resources=process_resources or {},
         )
 
         # Fetch latest Data object from database
