@@ -5,6 +5,7 @@ Resolwe Test Helpers and Decorators
 ===================================
 
 """
+
 import functools
 import json
 import os
@@ -22,8 +23,8 @@ from django.core.exceptions import ValidationError
 from django.test import override_settings, tag
 
 from resolwe.flow.utils import dict_dot, iterate_fields
+from resolwe.storage.connectors import connectors
 from resolwe.storage.models import FileStorage, StorageLocation
-from resolwe.storage.settings import STORAGE_LOCAL_CONNECTOR
 
 __all__ = (
     "check_installed",
@@ -98,8 +99,14 @@ def create_data_location(subpath=None):
     if subpath is None:
         subpath = file_storage.pk
 
+    data_connector = [
+        connector.name
+        for connector in connectors.for_storage("data")
+        if connector.mountable
+    ][0]
+
     StorageLocation.objects.create(
-        url=subpath, file_storage=file_storage, connector_name=STORAGE_LOCAL_CONNECTOR
+        url=subpath, file_storage=file_storage, connector_name=data_connector
     )
     return file_storage
 
@@ -267,11 +274,27 @@ def get_processes_from_tags(test):
     return slugs
 
 
+def ignore_in_tests(decorator):
+    """Skip a decorator when running tests.
+
+    This is useful for decorators like caching by id, which should not be
+    applied during testing, since ids are reused between tests.
+    """
+
+    def wrapper(wrapped):
+        if is_testing():
+            return wrapped
+        else:
+            return decorator(wrapped)
+
+    return wrapper
+
+
 def is_testing():
     """Return current testing status.
 
     This assumes that the Resolwe test runner is being used.
     """
-    from resolwe.test_helpers.test_runner import is_testing
+    from resolwe.test_helpers import is_testing
 
     return is_testing()
