@@ -2,21 +2,28 @@
 Django settings for running tests for Resolwe package.
 
 """
+
+import json
 import os
 import sys
 from distutils.util import strtobool
 from pathlib import Path
 
+from decouple import Csv, config
+
+from resolwe.__about__ import __version__
+
 PROJECT_ROOT = Path(__file__).parent.resolve()
 
 SECRET_KEY = "secret"
 
 DEBUG = True
 
-MIDDLEWARE_CLASSES = (
+MIDDLEWARE = (
     "django.contrib.sessions.middleware.SessionMiddleware",
     "django.middleware.common.CommonMiddleware",
     "django.contrib.auth.middleware.AuthenticationMiddleware",
+    "resolwe.auditlog.middleware.ResolweAuditMiddleware",
 )
 
 INSTALLED_APPS = (
@@ -27,7 +34,8 @@ INSTALLED_APPS = (
     "channels",
     # 'kombu.transport.django',  # required for Celery to work with Django DB.
     "rest_framework",
-    "guardian",
+    "drf_spectacular",
+    "django_filters",
     "versionfield",
     "resolwe",
     "resolwe.permissions",
@@ -35,6 +43,8 @@ INSTALLED_APPS = (
     "resolwe.storage",
     "resolwe.toolkit",
     "resolwe.test_helpers",
+    "resolwe.observers",
+    "resolwe.auditlog",
 )
 
 ROOT_URLCONF = "tests.urls"
@@ -56,9 +66,11 @@ TEMPLATES = [
 
 AUTHENTICATION_BACKENDS = (
     "django.contrib.auth.backends.ModelBackend",
-    "guardian.backends.ObjectPermissionBackend",
+    "resolwe.permissions.permissions.ResolwePermissionBackend",
 )
 
+DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
+
 ANONYMOUS_USER_NAME = "public"
 
 # Get the current Tox testing environment
@@ -84,34 +96,74 @@ DATABASES = {
     }
 }
 
+# The Redis database used by Django Channels.
 REDIS_CONNECTION = {
-    "host": "localhost",
+    "host": os.environ.get("RESOLWE_REDIS_HOST", "localhost"),
     "port": int(os.environ.get("RESOLWE_REDIS_PORT", 56379)),
     "db": int(os.environ.get("RESOLWE_REDIS_DATABASE", 1)),
+    "protocol": (os.environ.get("RESOLWE_REDIS_PROTOCOL", "redis")),
 }
+REDIS_CONNECTION_STRING = "{protocol}://{host}:{port}/{db}".format(**REDIS_CONNECTION)
 
 LISTENER_CONNECTION = {
     # Keys in the hosts dictionary are workload connector names. Currently
     # supported are 'local', 'kubertenes', 'celery' and 'slurm'.
-    "hosts": {"local": "172.17.0.1"},
+    "hosts": config(
+        "RESOLWE_LISTENER_SERVICE_HOST",
+        default='{"local": "172.17.0.1"}',
+        cast=json.loads,
+    ),
     "port": int(os.environ.get("RESOLWE_LISTENER_SERVICE_PORT", 53893)),
     "min_port": 50000,
     "max_port": 60000,
     "protocol": "tcp",
+    # Define the max number of commands listener can process simultaneously.
+    "max_concurrent_commands": config(
+        "RESOLWE_LISTENER_MAX_CONCURRENT_COMMANDS", cast=int, default=10
+    ),
 }
 
+# The IP address where listener is available from the communication container.
+# The setting is a dictionary where key is the name of the workload connector.
+COMMUNICATION_CONTAINER_LISTENER_CONNECTION = config(
+    "RESOLWE_COMMUNICATION_CONTAINER_LISTENER_CONNECTION",
+    default=json.dumps(LISTENER_CONNECTION["hosts"]),
+    cast=json.loads,
+)
+
+#: Add affinity to the Kubernetes jobs. Example:
+#: {"scheduling_class":
+#:   {"batch": {"nodegroup": ["compute"]}, "interactive": {"nodegroup": ["interactive"]}}
+#: The keys in the scheduling_class dictionary must correspond to the lower
+#: case name of the process scheduling class.
+FLOW_KUBERNETES_AFFINITY = None
+
+# Add the settings to overcommit the CPU and memory based on the process
+# scheduling type. Example:
+# {"batch": {"cpu": 0.8, "memory": 0.8}}
+# The default overcommitment factor is 0.8.
+FLOW_KUBERNETES_OVERCOMMIT = None
+
+FLOW_KUBERNETES_PULL_POLICY = config("RESOLWE_KUBERNETES_PULL_POLICY", default="Always")
+
+# The  ``KUBERNETES_DISPATCHER_CONFIG_LOCATION`` specifies where the
+# kubernetes workload connector reads the config from. The possible choices
+# are 'incluster' and 'kubectl'. The default value is 'incluster'.
+KUBERNETES_DISPATCHER_CONFIG_LOCATION = os.environ.get(
+    "RESOLWE_KUBERNETES_DISPATCHER_CONFIG", "incluster"
+)
+
+# Settings in OSX/Windows are different since Docker runs in a virtual machine.
 if sys.platform == "darwin":
     LISTENER_CONNECTION["hosts"]["local"] = "127.0.0.1"
+    COMMUNICATION_CONTAINER_LISTENER_CONNECTION = {"local": "127.0.0.1"}
 
 ASGI_APPLICATION = "resolwe.flow.routing.channel_routing"
 
 CHANNEL_LAYERS = {
     "default": {
         "BACKEND": "channels_redis.core.RedisChannelLayer",
-        "CONFIG": {
-            "hosts": [(REDIS_CONNECTION["host"], REDIS_CONNECTION["port"])],
-            "expiry": 3600,
-        },
+        "CONFIG": {"hosts": [REDIS_CONNECTION_STRING], "expiry": 3600},
     },
 }
 
@@ -126,7 +178,6 @@ STATIC_URL = "/static/"
 manager_prefix = os.environ.get("RESOLWE_MANAGER_REDIS_PREFIX", "resolwe.flow.manager")
 FLOW_MANAGER = {
     "REDIS_PREFIX": manager_prefix,
-    "REDIS_CONNECTION": REDIS_CONNECTION,
     "TEST": {
         "REDIS_PREFIX": manager_prefix + "-test",
     },
@@ -144,11 +195,6 @@ FLOW_DOCKER_IGNORE_PULL_ERRORS = strtobool(
 # Don't pull Docker images if set via the environment variable.
 FLOW_DOCKER_DONT_PULL = strtobool(os.environ.get("RESOLWE_DOCKER_DONT_PULL", "0"))
 
-# Disable SECCOMP if set via environment variable.
-FLOW_DOCKER_DISABLE_SECCOMP = strtobool(
-    os.environ.get("RESOLWE_DOCKER_DISABLE_SECCOMP", "0")
-)
-
 # Ensure all container images follow a specific format.
 FLOW_CONTAINER_VALIDATE_IMAGE = r".+:(?!latest)"
 
@@ -176,10 +222,22 @@ FLOW_PROCESSES_FINDERS = (
 
 FLOW_PROCESSES_RUNTIMES = ("resolwe.process.runtime.Process",)
 
+# Which processes to schedule (if not set, all).
+FLOW_PROCESSES_ALLOW_LIST = config(
+    "FLOW_PROCESSES_ALLOW_LIST", default="null", cast=json.loads
+)
+
+# Which processes to ignore (by default, none).
+FLOW_PROCESSES_IGNORE_LIST = config(
+    "FLOW_PROCESSES_IGNORE_LIST", default="null", cast=json.loads
+)
+
+FLOW_DOCKER_NETWORK = config("RESOLWE_DOCKER_NETWORK", default="bridge")
+
 FLOW_EXECUTOR = {
     "NAME": "resolwe.flow.executors.docker",
-    "REDIS_CONNECTION": REDIS_CONNECTION,
     "LISTENER_CONNECTION": LISTENER_CONNECTION,
+    "NETWORK": FLOW_DOCKER_NETWORK,
 }
 
 FLOW_DOCKER_AUTOREMOVE = False
@@ -196,19 +254,27 @@ REST_FRAMEWORK = {
     ),
     "DEFAULT_FILTER_BACKENDS": (
         "resolwe.permissions.filters.ResolwePermissionsFilter",
-        "django_filters.rest_framework.backends.DjangoFilterBackend",
+        "django_filters.rest_framework.DjangoFilterBackend",
         "resolwe.flow.filters.OrderingFilter",
     ),
     "EXCEPTION_HANDLER": "resolwe.flow.utils.exceptions.resolwe_exception_handler",
-    # Python<3.7 cannot parse iso-8601 formatted datetimes with tz-info form
-    # "+01:00" (DRF default). It can only parse "+0100" form, so we need to
-    # modify this setting. This will be fixed in Python3.7, where "+01:00" can
-    # be parsed by ``datetime.datetime.strptime`` syntax.
-    # For more, check "%z" syntax description in:
-    # https://docs.python.org/3.7/library/datetime.html#strftime-and-strptime-behavior
-    "DATETIME_FORMAT": "%Y-%m-%dT%H:%M:%S.%f%z",
+    "DATETIME_FORMAT": "iso-8601",
+    "DATE_FORMAT": "iso-8601",
+    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
 }
 
+
+# Schema generation
+
+SPECTACULAR_SETTINGS = {
+    "TITLE": "Resolwe",
+    "DESCRIPTION": "An open source dataflow package for Django.",
+    "VERSION": __version__,
+    "SERVE_INCLUDE_SCHEMA": False,
+    "ENUM_NAME_OVERRIDES": {"DataStatus": "resolwe.flow.models.Data.STATUS_CHOICES"},
+}
+
+
 # Time
 USE_TZ = True
 TIME_ZONE = "UTC"
@@ -261,6 +327,9 @@ LOGGING = {
         "standard": {
             "format": "%(asctime)s - %(levelname)s - %(name)s[%(process)s]: %(message)s",
         },
+        "auditlog": {
+            "format": "%(asctime)s - %(name)s[%(process)s]: %(message)s ; SID='%(session_id)s' RID='%(request_id)s'",
+        },
     },
     "handlers": {
         "console": {
@@ -268,6 +337,11 @@ LOGGING = {
             "level": CONSOLE_LEVEL,
             "formatter": "standard",
         },
+        "auditlog": {
+            "class": "logging.StreamHandler",
+            "level": "INFO",
+            "formatter": "auditlog",
+        },
         "file": {
             "class": "logging.handlers.RotatingFileHandler",
             "filename": debug_file_path,
@@ -280,5 +354,9 @@ LOGGING = {
             "handlers": default_logger_handlers,
             "level": "DEBUG",
         },
+        "auditlog": {
+            "handlers": ["auditlog"],
+            "level": "INFO",
+        },
     },
 }
